@@ -18,9 +18,22 @@ class Target(object):
     server = None
 
     def __init__(self):
+        import graphlab.canvas.views.job_session
+        import graphlab.canvas.views.data_objects
+        import graphlab.canvas.views.ps_session
+        from graphlab.deploy import jobs
+        from graphlab.deploy import predictive_services
         self.state = graphlab.canvas.state.State()
         # TODO server is not necessary in static IPython Notebook
         self.server = graphlab.canvas.server.Server(self.state)
+        # add data objects to left nav
+        self.add_variable(('Data',), graphlab.canvas.views.data_objects.DataObjectsView())
+        # add predictive service dashboard to left nav
+        PSView = graphlab.canvas.views.ps_session.PredictiveServiceList(predictive_services)
+        self.add_variable(PSView.get_temporary_name(), PSView)
+        # add job dashboard to left nav
+        jobsView = graphlab.canvas.views.job_session.ScopedSessionView(jobs)
+        self.add_variable(jobsView.get_temporary_name(), jobsView)
 
     # methods
     def show(self, variable=None):
@@ -83,13 +96,13 @@ class IPythonTarget(Target):
         """ % css.replace('"', '\\"').replace('\n', '')
         return js
 
-    def __makeJS(self, data, view):
+    def __makeJS(self, data, viewFile, viewComponent):
         """
         Generate a JavaScript snippet that will load Canvas in an output cell
         of IPython Notebook.
         """
         # requirejs dependencies for our code snippet
-        dependencies = ['react', view]
+        dependencies = ['react', viewFile]
         # dependency files (we will embed this code in the output cell)
         # order matters! these files must be listed in dependency order
         dependencyFiles = [
@@ -99,17 +112,20 @@ class IPythonTarget(Target):
             'state.js',
             'taskqueue.js',
             'plot_frequency.js',
+            'plot_bar.js',
             'plot_box.js',
             'plot_boxes.js',
+            'plot_heatmap.js',
             'plot_quantilehist.js',
             'plot_graph.js',
+            'plot_scatter.js',
             'plots.js',
+            'bar.js',
+            'heatmap.js',
+            'scatter.js',
             'sframe.js',
             'sarray.js',
-            'sgraph.js',
-            'task.js',
-            'job.js',
-            'session.js',
+            'sgraph.js'
         ]
         dependencyFiles = string.join(map(IPythonTarget.__readFile, dependencyFiles), '\n')
         # Require paths must point to real URLs (not localhost or a file path)
@@ -120,12 +136,12 @@ class IPythonTarget(Target):
         # on the page, which it is in IPython Notebook, then react MUST be loaded through
         # requirejs, which means it has to come from a URL).
         requirePaths = json.dumps({
-            "react": "//fb.me/react-with-addons-0.10.0.min",
+            "react": "//fb.me/react-with-addons-0.11.1.min",
             "d3": "//d3js.org/d3.v3.min",
-            "dagre": "//cpettitt.github.io/project/dagre/latest/dagre.min",
+            "dagre": "//cpettitt.github.io/project/dagre/v0.4.6/dagre.min",
             "topojson": "//d3js.org/topojson.v1.min",
             "vega": "//trifacta.github.io/vega/vega.min",
-            "bowser": "//www.javascriptoo.com/application/html/js/ded/bowser/bowser.min",
+            "bowser": "//www.javascriptoo.com/application/html/js/ded/bowser/bowser.min"
         })
         # Code snippet to include dependency JS files, CSS styles, and a require call to run Canvas
         js = u"""
@@ -154,20 +170,24 @@ class IPythonTarget(Target):
                     e = element[0];
                 }
 
-                require(%(dependencies)s, function(React, Component) {React.renderComponent(Component.View(%(data)s), e);});
+                require(%(dependencies)s, function(React, Component) {React.renderComponent(Component.%(view)s(%(data)s), e);});
             })();
         """ % {
             "dependencies": dependencies,
             "dependencyFiles": dependencyFiles,
             "data": data,
             "css": self.__makeCSS(),
-            "requirePaths": requirePaths
+            "requirePaths": requirePaths,
+            "view": viewComponent
         }
         return js
 
     def show(self, variable=None):
         super(IPythonTarget, self).show(variable)
-        (name, view) = self.state.get_selected_variable()
+        selected = self.state.get_selected_variable()
+        if selected is None:
+            raise BaseException('No object is currently selected for viewing in Canvas. Call .show on a GraphLab object to show output in IPython Notebook.')
+        (name, view) = selected
         assert(type(name) == tuple)
         assert(isinstance(view, graphlab.canvas.views.base.BaseView))
 
@@ -181,5 +201,6 @@ class IPythonTarget(Target):
 
         import IPython
         return IPython.core.display.Javascript(
-            data=self.__makeJS(json.dumps(data), view.objectType.lower())
+            data=self.__makeJS(json.dumps(data), view.get_js_file(), view.get_js_component()),
+            css=['//cdnjs.cloudflare.com/ajax/libs/font-awesome/4.1.0/css/font-awesome.min.css']
         )
