@@ -66,6 +66,7 @@ def show(variable=None):
     """
     if variable is not None:
         get_target().state.set_selected_variable(variable)
+        variable.validate_js_component_name(variable.get_js_component())
     return get_target().show()
 
 def _get_id(ref):
@@ -95,7 +96,7 @@ def _get_id(ref):
     if isinstance(ref, (graphlab.deploy._artifact.Task,
                         graphlab.deploy._artifact.Pipeline,
                         graphlab.deploy._artifact.RuntimeTask,
-                        graphlab.deploy._context.Environment,
+                        graphlab.deploy.environment.Environment,
                         graphlab.deploy._job.Job)):
         return long(hash(str(ref.name)))
 
@@ -107,3 +108,26 @@ def _same_object(ref1, ref2):
     if type(ref1) != type(ref2):
         return False
     return _get_id(ref1) == _get_id(ref2)
+
+def _profile(func):
+    import functools
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        import cProfile, pstats, StringIO, tempfile
+        import time
+        pr = cProfile.Profile()
+        pr.enable()
+        time1 = time.time()
+        try:
+            return func(*args, **kwargs)
+        finally:
+            time2 = time.time()
+            pr.disable()
+            s = StringIO.StringIO()
+            ps = pstats.Stats(pr, stream=s).sort_stats('cumulative')
+            ps.print_stats()
+            with tempfile.NamedTemporaryFile(delete=False) as f:
+                f.write(s.getvalue())
+                print 'function took %0.3f ms' % ((time2-time1)*1000.0)
+                print 'log file at %s' % f.name
+    return wrapper
