@@ -1,4 +1,5 @@
 import array
+import datetime
 import json
 import re
 import textwrap
@@ -18,6 +19,45 @@ def _encode_value(value):
     if value is None:
         return value
 
+    # convert datetime to str
+    if isinstance(value, datetime.datetime):
+        # return, don't go through truncation
+        return str(value)
+
+    # represent image as base64-encoded bytes
+    import binascii
+    from graphlab.data_structures.image import Image
+    if isinstance(value, Image):
+        image_format = None
+        if value._format_enum == 0:
+            image_format = 'jpeg'
+        elif value._format_enum == 1:
+            image_format = 'png'
+        elif value._format_enum == 2:
+            image_format = 'raw'
+        if image_format is not None:
+            ret = {
+                'type': 'image',
+                'width': value._width,
+                'height': value._height,
+                'channels': value._channels,
+                'format': image_format,
+                'id': id(value)
+            }
+            if image_format in ('jpeg', 'png'):
+                ret.update({
+                    'value': 'image/%s;base64,%s' % (image_format, binascii.b2a_base64(value._image_data))
+                })
+            elif image_format == 'raw':
+                ret.update({
+                    'value': list(value._image_data)
+                })
+            return ret
+
+        # fallback case for images the browser does not know how to display
+        # just convert to str and treat like any other type
+        value = str(value)
+
     # convert strings to unicode (assumes utf-8 encoding, replaces invalid
     # characters with ?
     if isinstance(value, str):
@@ -67,7 +107,7 @@ class BaseView(object):
         self.objectType = type(obj).__name__
         self.handlers = {}
         self.children = {}
-        self.params = params
+        self.params = {} if params is None else params
 
     def _handle_request(self, method, url, handler):
         """
@@ -193,6 +233,14 @@ class BaseView(object):
         """
         return 'View';
 
+    def get_js_components(self):
+        """
+        Override this method to provide a list of possible components.
+        Each will be shown as a tab to the user.
+        The default is to show just the selected component.
+        """
+        return [self.get_js_component()]
+
     def get_temporary_name(self):
         """
         Override this method to provide a specific temporary name (tuple) 
@@ -200,3 +248,10 @@ class BaseView(object):
         called when the state cannot find the name of the selected variable.
         """
         return ('<temporary %s>' % type(self.obj).__name__,)
+
+    def validate_js_component_name(self, name):
+        """
+        Override to raise an exception if an invalid view name is passed.
+        """
+        if not(name in self.get_js_components()):
+            raise ValueError('View "%s" does not exist on this object.' % name)
