@@ -37,6 +37,9 @@ class SGraphView(graphlab.canvas.views.base.BaseView):
 
     def get_js_file(self):
         return 'sgraph'
+    
+    def get_js_component(self):
+        return 'View'
 
     def __exceeds_vertex_limit(self):
         ## Error trapping
