@@ -24,122 +24,63 @@ from graphlab.cython.cy_ipc import get_public_secret_key_pair
 from graphlab_util.config import DEFAULT_CONFIG as config
 from graphlab.connect import _get_metric_tracker
 
+
 CONFIG_SECTION = 'AWS'
 DEFAULT_CIDR_RULE = '0.0.0.0/0'  # Open to the entire internet.
 DEFAULT_INSTANCE_TYPE = 'm3.xlarge'
-GRAPHLAB_PORT_MIN_NUM = 9000
-GRAPHLAB_PORT_MAX_NUM = 9004
 JSON_BLOB_PATH_FORMAT = "/gl_api/api/v1/cloud/aws/instances/%s?client_version=%s&region=%s"
 GRAPHLAB_NAME = 'GraphLab'
 
+# Port 9000 & 9001 - used when running in server mode: only server running with external client.
+# Port 9002 & 9003 - used for metrics (deprecated).
+# Port 9004 - for jobs/tasks.
+# Port 9005 & 9006 - used by predictive service (webservice & reddis).
+GRAPHLAB_PORT_MIN_NUM = 9000
+GRAPHLAB_PORT_MAX_NUM = 9006
+ADDITIONAL_REDIS_PORT = 19006
+
 # Swallow all boto logging, except critical issues
 logging.getLogger('boto').setLevel(logging.CRITICAL)
 
 __LOGGER__ = logging.getLogger(__name__)
 
-class _EC2(glserver.RemoteServer):
-    """
-    This class is responsible for starting up an EC2 instance.
-    """
-    def __init__(self, instance_type, region, CIDR_rule, auth_token, product_key, security_group_name, 
-                 tags, public_key, secret_key, user_data, credentials):
-        self.logger = __LOGGER__
+
+class _Ec2Instance:
+    def __init__(self, public_dns_name, instance_id, instance, region):
+        self.public_dns_name = public_dns_name
+        self.instance_id = instance_id
+        self.instance = instance
         self.region = region
+
+    def stop(self):
+        _stop_instances([self.instance_id], self.region)
+
+
+class _Ec2GraphLabServer(glserver.RemoteServer):
+    def __init__(self, instance_type, region, CIDR_rule, auth_token, security_group_name, tags,
+                 public_key, secret_key):
+        self.logger = __LOGGER__
         self.auth_token = auth_token
-        
+
         _get_metric_tracker().track(('ec2.instance_type.%s' % instance_type), value=1)
         _get_metric_tracker().track(('ec2.region.%s' % region), value=1)
 
-        try:
-            conn = boto.ec2.connect_to_region(self.region, **credentials)
-
-            # Does the security group already exist?
-            security_group = None
-            for sg in conn.get_all_security_groups():
-                if(security_group_name == sg.name):
-                    security_group = sg
-                    break
-
-        except boto.exception.EC2ResponseError as e:
-            # This is the first AWS contact, so the most likely cause is environment variables 
-            # aren't set.
-            self.logger.debug('EC2 Response Error: %s' % e)
-            raise Exception('EC2 response error. Please verify your AWS credentials. To configure your AWS credentials use graphlab.aws.set_credentials or set the AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY environment variables')
-        except boto.exception.NoAuthHandlerFound as e:
-            raise Exception('AWS configuration not found. Please configure your AWS credentials using graphlab.aws.set_credentials or by setting AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY environment variables')
-
-        # If security group doesn't exist, create it.
-        if security_group is None:
-            security_group = conn.create_security_group(security_group_name,
-                                                        'Only open ports needed for GraphLab Server')
-
-        # Autorize the user CIDR rules for the ports we need
-        try:
-            security_group.authorize('tcp', GRAPHLAB_PORT_MIN_NUM, GRAPHLAB_PORT_MAX_NUM, CIDR_rule)
-        except EC2ResponseError as e:
-            if not "<Code>InvalidPermission.Duplicate</Code>" in str(e):
-                raise
-
-        if (config.mode != 'PROD' and 'GRAPHLAB_TEST_AMI_ID' in os.environ and 'GRAPHLAB_TEST_ENGINE_URL' in os.environ):
-            # unit-test mode, don't involve webservice to retrieve AMI, instead use environment variables
-            ami_id = os.environ['GRAPHLAB_TEST_AMI_ID']
-            engine_url = os.environ['GRAPHLAB_TEST_ENGINE_URL']
-            self.logger.info("UNIT mode, using AMI: '%s' and engine url: '%s' when launching EC2 instance." % (ami_id, engine_url))
-            json_blob = json.loads('{}')
-            json_blob['ami_id'] = ami_id
-            json_blob['engine_url'] = engine_url
-            json_blob['os_url'] = ''
-        else:
-            # Get the info to start a EC2 from the GraphLab Server
-            json_blob_path = JSON_BLOB_PATH_FORMAT % (instance_type, config.version, self.region)
-            json_blob_url = config.graphlab_server + json_blob_path
-
-            try:
-                # set specific timeout for this web service request, lots of time spent in SSL negotiation
-                graphlab_server_response = urllib2.urlopen(json_blob_url, timeout=10)
-                json_blob = json.loads(graphlab_server_response.read())
-            except:
-                raise Exception("Unable to successfully retrieve correct EC2 image to launch for this version. This could be a temporary problem. Please try again in a few minutes. If the problem persists please contact support@graphlab.com")
-
-        self.logger.debug("web service return: %s" % json_blob)
-
-        if json_blob['ami_id'] is None:
-            raise Exception("Unable to successfully retrieve correct EC2 image to launch. Please try again later or contact support@graphlab.com. Error received:'%s'" % json_blob['message'])
-        ami_id = json_blob['ami_id']
-
+        # Configure user data that will be sent to EC2 instance.
+        user_data = {}
         if self.auth_token is not None:
-            json_blob['auth_token'] = self.auth_token
+            user_data['auth_token'] = self.auth_token
         else:
-            json_blob['auth_token'] = ''
-
+            user_data['auth_token'] = ''
         if secret_key != '':
-            json_blob['secret_key'] = secret_key
+            user_data['secret_key'] = secret_key
 
-        # set product key in json_blob
-        if product_key is None:
-            product_key = ''
-        json_blob['product_key'] = product_key
-
-        user_data.update(json_blob)
-
-        run_instances_args =  {
-            'security_group_ids' : [ security_group_name ],
-            'user_data' : json.dumps(user_data),
-            'instance_type' : instance_type
-        }
-
-        if (config.mode != 'PROD' and 'GRAPHLAB_TEST_EC2_KEYPAIR' in os.environ):
-            keypair = os.environ['GRAPHLAB_TEST_EC2_KEYPAIR']
-            self.logger.info("UNIT mode, using keypair: '%s' when launching EC2 instance" % (keypair))
-            run_instances_args['key_name'] = keypair
-
-        # Launch the EC instance
-        response = conn.run_instances(ami_id, **run_instances_args)
-
-        if(not(len(response.instances) == 1)):
-            raise Exception('Could not successfully start EC2 instance.')
-
-        self.instance = response.instances[0]
+        # Launch the EC2 instance.
+        ec2_instance = _ec2_factory(instance_type=instance_type, region=region, CIDR_rule=CIDR_rule,
+                                    security_group_name=security_group_name, tags=tags,
+                                    user_data = user_data)
+        self.instance = ec2_instance.instance
+        self.public_dns_name = self.instance.public_dns_name
+        self.region = ec2_instance.region
 
         # If auth_token is not set by the input argument, the EC2 instance
         # will use its instance id as the default auth_token, so we reset
@@ -147,33 +88,17 @@ class _EC2(glserver.RemoteServer):
         if not self.auth_token:
             self.auth_token = self.instance.id
 
-        self.logger.info("Launching an %s instance in the %s availability zone, with id: %s. You will be responsible for the cost of this instance." % (self.instance.instance_type, self.instance.placement, self.instance.id))
-
-        # Wait for the host to start up
-        while not self.instance.update() == 'pending':
-            time.sleep(1)
-        while self.instance.update() == 'pending':
-            time.sleep(1)
-
         if self.instance.state == 'running':
             self.instance_id = self.instance.id
             server_addr = 'tcp://' + self.instance.ip_address + ':' + str(GRAPHLAB_PORT_MIN_NUM)
             self.logger.debug("EC2 instance is now running, attempting to connect to engine.")
-            super(self.__class__, self).__init__(server_addr, auth_token=self.auth_token, 
+            super(self.__class__, self).__init__(server_addr, auth_token=self.auth_token,
                                                  public_key=public_key)
         else:   # Error case
             self.logger.error('Could not startup EC2 instance.')
             if(hasattr(self.instance, 'id')):
                 self.stop()
-            return 
-        
-        self.public_dns_name = self.instance.public_dns_name
-
-        # Add tags to this instance.
-        if(tags is None):
-            tags = {}
-        tags[GRAPHLAB_NAME] = ''
-        conn.create_tags(self.instance.id, tags)
+            return
 
 
     def get_auth_token(self):
@@ -181,23 +106,7 @@ class _EC2(glserver.RemoteServer):
 
     def stop(self):
         _get_metric_tracker().flush()
-        self.logger.info('Terminating EC2 instance, id: %s.' % (self.instance_id))
-        try:
-            conn = boto.ec2.connect_to_region(self.region)
-            response = conn.terminate_instances(instance_ids=[self.instance_id])
-            if(not(len(response) == 1)):
-                self.logger.error('Invalid response from EC2. Unable to terminate host. Please terminate '
-                                'using the AWS EC2 Console.')
-                return
-            if response[0].update() == 'shutting-down':
-                while response[0].update() == 'shutting-down':
-                    time.sleep(1)
-            else:
-                self.logger.error('Invalid response from EC2. Unable to terminate host. Please terminate '
-                                'using the AWS EC2 Console.')
-        except:
-            self.logger.error('Invalid response from EC2. Unable to terminate host. Please terminate '
-                              'using the AWS EC2 Console.')
+        _stop_instances([self.instance_id], self.region)
 
 
 def _get_region_from_config(config_path = (os.path.join(os.path.expanduser("~"), ".graphlab", "config"))):
@@ -234,9 +143,9 @@ def get_credentials():
     """
 
     if (not 'AWS_ACCESS_KEY_ID' in os.environ):
-        raise KeyError('No access key found. Please set the environment variable AWS_ACCESS_KEY_ID, or using graphlab.aws.set_credential()')
+        raise KeyError('No access key found. Please set the environment variable AWS_ACCESS_KEY_ID, or using graphlab.aws.set_credentials()')
     if (not 'AWS_SECRET_ACCESS_KEY' in os.environ):
-        raise KeyError('No secret key found. Please set the environment variable AWS_SECRET_ACCESS_KEY, or using graphlab.aws.set_credential()')
+        raise KeyError('No secret key found. Please set the environment variable AWS_SECRET_ACCESS_KEY, or using graphlab.aws.set_credentials()')
     return (os.environ['AWS_ACCESS_KEY_ID'], os.environ['AWS_SECRET_ACCESS_KEY'])
 
 
@@ -273,7 +182,7 @@ def launch_EC2(instance_type=DEFAULT_INSTANCE_TYPE, region=None, CIDR_rule=None,
     security_group : string, optional
         The name of the security group for the EC2 instance to use.
 
-    tags : dict, optional    
+    tags : dict, optional
         A dictionary containing the name/value tag pairs to be assigned to the
         instance. If you want to create only a tag name, the value for that tag
         should be the empty string (i.e. ''). In addition to these specified
@@ -311,9 +220,8 @@ def launch_EC2(instance_type=DEFAULT_INSTANCE_TYPE, region=None, CIDR_rule=None,
         (client_public_key, client_secret_key) = get_public_secret_key_pair()
         __LOGGER__.debug('Launching server with public key: %s' % server_public_key)
 
-    server = _ec2_factory(instance_type=instance_type, region=region, CIDR_rule=CIDR_rule, auth_token=auth_token,
-                          security_group=security_group, tags=tags, server_public_key=server_public_key,
-                          server_secret_key=server_secret_key)
+    server = _Ec2GraphLabServer(instance_type, region, CIDR_rule, auth_token, security_group, tags,
+                                server_public_key, server_secret_key)
 
     try:
         # Once the EC2 instance starts up, the client still need to wait for graphlab engine to start up.
@@ -321,10 +229,13 @@ def launch_EC2(instance_type=DEFAULT_INSTANCE_TYPE, region=None, CIDR_rule=None,
         # in other AWS regions (since longer latency and longer download from S3 for engine binary)
         server.start(num_tolerable_ping_failures=120)
     except Exception as e:
-        __LOGGER__.error("Unable to successfully connect to GraphLab Server on EC2 instance: '%s'. Please check AWS Console to make sure any EC2 instances launched have been terminated." % e)
+        __LOGGER__.error("Unable to successfully connect to GraphLab Server on EC2 instance: '%s'."
+                         " Please check AWS Console to make sure any EC2 instances launched have"
+                         " been terminated." % e)
         server.stop()
         raise e
 
+    # Create the client
     num_tolerable_ping_failures = 3
     client = Client([], server.get_server_addr(), num_tolerable_ping_failures, public_key=client_public_key,
                     secret_key=client_secret_key, server_public_key=server_public_key)
@@ -369,17 +280,17 @@ def list_instances(region='us-west-2'):
     Instance IDs for all active GraphLab EC2 hosts. Returns instance IDs for all
     active GraphLab EC2 hosts (for the specified region), not just the instance
     ID associated with the current python session.
-    
+
     Parameters
     ----------
-    region : string, optional    
+    region : string, optional
         The `AWS region
         <http://docs.aws.amazon.com/general/latest/gr/rande.html#ec2_region>`_
         in which to list instances. Default is 'us-west-2'.
 
     Returns
     -------
-    out : list[string]    
+    out : list[string]
         A list of AWS instance IDs corresponding to active instances in the
         specified region.
 
@@ -412,15 +323,16 @@ def status():
     Examples
     --------
     >>> # With an EC2 instance running.
-    >>> graphlab.aws.status()    
+    >>> graphlab.aws.status()
     u'running'
 
     >>> # Without an EC2 instance running.
     >>> graphlab.aws.status()
+    >>> 'No instance running.'
     """
 
     server = glconnect.get_server()
-    if server is None or not isinstance(server, _EC2):
+    if server is None or not isinstance(server, _Ec2GraphLabServer):
         return 'No instance running.'
     aws_connection = boto.ec2.connect_to_region(server.region)
     status = aws_connection.get_all_instance_status(instance_ids=[server.instance_id])
@@ -438,57 +350,223 @@ def terminate_EC2():
 
     Examples
     --------
-    >>> graphlab.aws.terminate_EC2() 
+    >>> graphlab.aws.terminate_EC2()
     """
     server = glconnect.get_server()
-    if server is not None and isinstance(server, _EC2):
+    if server is not None and isinstance(server, _Ec2GraphLabServer):
         __LOGGER__.debug("Stopping EC2 instance.")
         glconnect.stop()
     else:
         __LOGGER__.info("No EC2 instance to stop.")
 
 
-def _ec2_factory(instance_type, region=None, CIDR_rule=None, auth_token=None, security_group=None,
-                 tags=None, server_public_key='', server_secret_key='', user_data = {},
-                 credentials = {}):
+def _stop_instances(instance_id_list, region, credentials = {}):
+    __LOGGER__.info("Terminating EC2 host(s) %s in %s" % (instance_id_list, region))
+
+    # Hopefully we will not need this string
+    ERROR_MSG = 'Invalid response from EC2. Unable to terminate host(s). Please terminate using the ' \
+        'AWS EC2 Console.'
 
-    # get and validate the product / registration key
     try:
-        product_key = graphlab.product_key.get_product_key()
-    except KeyError as k:
-        __LOGGER__.error(k.message)
-        return
+        # Shut the host(s) down.
+        conn = boto.ec2.connect_to_region(region, **credentials)
+        response = conn.terminate_instances(instance_ids = instance_id_list)
 
+        # Sanity check response.
+        if(not(len(response) == len(instance_id_list))):
+            raise Exception(ERROR_MSG)
+        for r in response:
+            status = r.update()
+            if status not in ['shutting-down', 'terminated']:
+                raise Exception(ERROR_MSG)
+    except:
+        raise Exception(ERROR_MSG)
+
+
+def _ec2_factory(instance_type, region=None, CIDR_rule=None, security_group_name=None, tags=None,
+                 user_data = {}, credentials = {}, ami_service_parameters = {}, num_hosts = 1):
+    '''
+    This function does everything necessary to bring up EC2 host(s): create a security group (if
+    nessary), determine arguments to start up the EC2 instance (i.e. AMI id and user data),
+    actually start up the EC2 instance, wait for it, and applies AWS tags.
+    '''
+
+    # Get and validate the product key
+    product_key = graphlab.product_key.get_product_key()
     if (not graphlab.product_key.is_product_key_valid(product_key)):
         __LOGGER__.error("Product Key validation failed, please confirm your product key is correct. "
                          "If you believe this key to be valid, please contact support@graphlab.com")
         return
 
-    # Determine region
+    # Set default values for parameters.
     if(region is None):
         region = _get_region_from_config()
         if(region is None):
             region = 'us-west-2'
-
     # If a CIDR rule is specified, a security group must also be specified.
     if CIDR_rule is not None and security_group is None:
         __LOGGER__.error('If you specify a CIDR rule, you must also specify an existing security group.')
         return
     # If no security group is specified (and no CIDR rule), use a default value for the security group name.
-    elif security_group is None:
-        security_group = GRAPHLAB_NAME
-
+    elif security_group_name is None:
+        security_group_name = GRAPHLAB_NAME
     if CIDR_rule is None:
         CIDR_rule = DEFAULT_CIDR_RULE
 
-    # By default the EC2 instance will use its instance id as the auth_token.
+    # Setup/get the security group.
     try:
-        server = _EC2(instance_type, region, CIDR_rule, auth_token, product_key, security_group,
-                      tags, server_public_key, server_secret_key, user_data, credentials)    # Launches the EC2 instance
-    except Exception as e:
-      __LOGGER__.error("Unable to launch EC2 instance: '%s'. Please check AWS Console to make sure any EC2 instances launched have been terminated." % e)
-      # since EC2 launch could be part of a script, we want to raise the exception so that remaining code
-      # stops executing (otherwise a local server would be created and the code would continue locally)
-      raise e
+        conn = boto.ec2.connect_to_region(region, **credentials)
+        # Does the security group already exist?
+        security_group = None
+        for sg in conn.get_all_security_groups():
+            if(security_group_name == sg.name):
+                security_group = sg
+                break   # found it
+    except boto.exception.EC2ResponseError as e:
+        # This is the first AWS contact, so the most likely cause is env vars aren't set.
+        __LOGGER__.debug('EC2 Response Error: %s' % e)
+        raise Exception('EC2 response error. Please verify your AWS credentials. To configure your '
+                        'AWS credentials use graphlab.aws.set_credentials or set the '
+                        'AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY environment variables'
+                        'Exception: %s' % e)
+    except boto.exception.NoAuthHandlerFound as e:
+            raise Exception('AWS configuration not found. Please configure your AWS credentials '
+                            'using graphlab.aws.set_credentials or by setting AWS_ACCESS_KEY_ID '
+                            'and AWS_SECRET_ACCESS_KEY environment variables')
 
-    return server
+    # If security group doesn't exist, create it.
+    if security_group is None:
+        security_group = conn.create_security_group(security_group_name,
+                                                    'Only open ports needed for GraphLab Server')
+
+    # Configure the security group
+    try:
+        security_group.authorize('tcp', 443, 443, CIDR_rule)
+        security_group.authorize('tcp', 80, 80, CIDR_rule)
+        security_group.authorize('tcp', ADDITIONAL_REDIS_PORT, ADDITIONAL_REDIS_PORT, CIDR_rule)
+        security_group.authorize('tcp', GRAPHLAB_PORT_MIN_NUM, GRAPHLAB_PORT_MAX_NUM, CIDR_rule)
+    except EC2ResponseError as e:
+        if not "<Code>InvalidPermission.Duplicate</Code>" in str(e):
+            raise e
+
+    # Determine AMI id, as well as engine and os URL
+    if (config.mode != 'PROD' and 'GRAPHLAB_TEST_AMI_ID' in os.environ and 'GRAPHLAB_TEST_ENGINE_URL' in os.environ):
+        # unit-test mode, don't involve webservice to retrieve AMI, instead use environment variables
+        ami_id = os.environ['GRAPHLAB_TEST_AMI_ID']
+        engine_url = os.environ['GRAPHLAB_TEST_ENGINE_URL']
+        __LOGGER__.info("UNIT mode, using AMI: '%s' and engine url: '%s' when launching EC2 instance." % (ami_id, engine_url))
+        json_blob = json.loads('{}')
+        json_blob['ami_id'] = ami_id
+        json_blob['engine_url'] = engine_url
+        json_blob['os_url'] = ''
+    else:
+        # Get the info to start a EC2 from the GraphLab Server
+        json_blob_path = JSON_BLOB_PATH_FORMAT % (instance_type, config.version, region)
+        for (param_name, param_value) in ami_service_parameters.items():
+            json_blob_path += "&%s=%s" % (str(param_name), str(param_value))
+        json_blob_url = config.graphlab_server + json_blob_path
+
+        try:
+            # set specific timeout for this web service request, lots of time spent in SSL negotiation
+            graphlab_server_response = urllib2.urlopen(json_blob_url, timeout=10)
+            json_blob = json.loads(graphlab_server_response.read())
+        except:
+            raise Exception('Unable to successfully retrieve correct EC2 image to launch for this '
+                            'version. This could be a temporary problem. Please try again in a few '
+                            'minutes. If the problem persists please contact support@graphlab.com')
+    __LOGGER__.debug("web service return: %s" % json_blob)
+
+    if 'ami_id' not in json_blob or json_blob['ami_id'] is None:
+        raise Exception("Unable to successfully retrieve correct EC2 image to launch. Please try "
+                        "again later or contact support@graphlab.com. Error received:'%s'"
+                        % json_blob['message'])
+    ami_id = json_blob['ami_id']
+
+    # Add json_blob to user_data and set the product key
+    user_data.update(json_blob)
+    user_data['product_key'] = product_key
+
+    # Check for testing override of os_url param.
+    if (config.mode != 'PROD' and 'GRAPHLAB_TEST_OS_URL' in os.environ):
+        user_data['os_url'] = os.environ['GRAPHLAB_TEST_OS_URL']
+
+    run_instances_args =  {
+        'security_group_ids' : [ security_group_name ],
+        'user_data' : json.dumps(user_data),
+        'instance_type' : instance_type
+    }
+
+    if num_hosts != 1:
+        run_instances_args['min_count'] = num_hosts
+        run_instances_args['max_count'] = num_hosts
+
+    # If this isn't prod, may not to set SSH key for debugging.
+    if 'GRAPHLAB_TEST_EC2_KEYPAIR' in os.environ:
+        keypair = os.environ['GRAPHLAB_TEST_EC2_KEYPAIR']
+        __LOGGER__.info("UNIT mode, using keypair: '%s' when launching EC2 instance" % (keypair))
+        run_instances_args['key_name'] = keypair
+
+    # Actually launch the EC2 instance(s) and wait for them to start running.
+    instances = None
+    try:
+        response = conn.run_instances(ami_id, **run_instances_args)
+        instances = response.instances
+        if(len(response.instances) != num_hosts):
+            raise Exception
+
+        # Report
+        for i in instances:
+            __LOGGER__.info("Launching an %s instance in the %s availability zone, with id: %s."
+                            " You will be responsible for the cost of this instance."
+                            % (i.instance_type, i.placement, i.id))
+
+        # Wait for all host(s) to say they're done starting up.
+        while True:
+            try:
+                for i in instances:
+                    # Rarely an instance can a reach temp state before going into pending. We check for
+                    # 'running' right away to make unit tests work.
+                    while not i.update() in ['pending', 'running', 'failed']:
+                        time.sleep(1)
+                    while i.update() == 'pending':
+                        time.sleep(1)
+                    if i.update() == 'failed':
+                        raise RuntimeError("Instance %s startup failed" % i.id)
+                break
+            except EC2ResponseError as e:
+                # EC2 is eventual consistence so sometimes it complains that it
+                # cannot find the instance, in that case, we will retry
+                __LOGGER__.info("Ignoring EC2ResponseError: %s" % e.message)
+
+    except Exception as e:
+        if instances:
+            _stop_instances([i.id for i in instances] , self.region)
+        raise Exception("Unable to launch EC2 instance: '%s'. Please check AWS Console to make "
+                         " sure any EC2 instances launched have been terminated." % e)
+
+    # Add tags to this instance(s).
+    if(tags is None):
+        tags = {}
+    tags[GRAPHLAB_NAME] = ''
+    for i in instances:
+        conn.create_tags(i.id, tags)
+
+    results = []
+    for i in instances:
+        results.append(_Ec2Instance(i.public_dns_name, i.id, i, region))
+
+    if num_hosts == 1:
+        # for backward compatibility
+        return results[0]
+    return results
+
+
+def _get_ec2_instances(ec2_id_list, region, aws_credentials = {}):
+    conn = boto.ec2.connect_to_region(region, **aws_credentials)
+    response = conn.get_all_instances(instance_ids = ec2_id_list)
+
+    results = []
+    for reservation in response:
+        results += reservation.instances
+
+    return results
