@@ -78,7 +78,19 @@ def launch(server_addr=None, server_bin=None, server_log=None, auth_token=None,
         __LOGGER__.error(k.message)
         return
 
-    if (not graphlab.product_key.is_product_key_valid(product_key)):
+    # product key check. This may fail if there are issues running
+    # the unity_server binary
+    try:
+      product_key_good = graphlab.product_key.is_product_key_valid(product_key)
+    except RuntimeError as r:
+      __LOGGER__.error("Fatal error. The unity_server process cannot be started. There may have been an "
+                       "issue during installation of graphlab-create. Please uninstall graphlab-create "
+                       "and reinstall it, looking for errors that may occur during installation. "
+                       "If the problem persists, please contact support@graphlab.com.")
+      return
+
+    # verify product key is good
+    if (not product_key_good):
         __LOGGER__.error("Product Key validation failed, please confirm your product key is correct. "
                          "If you believe this key to be valid, please contact support@graphlab.com")
         return
@@ -121,7 +133,6 @@ def launch(server_addr=None, server_bin=None, server_log=None, auth_token=None,
 
     assert is_connected()
 
-
 def _get_server_type(server_addr):
     """
     Returns the server type in one of the {LOCAL_SERVER_TYPE, REMOTE_SERVER_TYPE},
@@ -222,21 +233,25 @@ def get_unity():
     if not is_connected():
         launch()
     assert is_connected(), "Cannot connect to GraphLab Server"
-    return UnityGlobalProxy(__CLIENT__)
+    return __UNITY_GLOBAL_PROXY__
 
 
 def _assign_server_and_client(server, client):
     """
     Helper function to assign the global __SERVER__ and __CLIENT__ pair.
     """
-    global __SERVER__, __CLIENT__
+    global __SERVER__, __CLIENT__, __UNITY_GLOBAL_PROXY__
     __SERVER__ = server
     __CLIENT__ = client
+    __UNITY_GLOBAL_PROXY__ = UnityGlobalProxy(__CLIENT__)
     server.get_logger().debug('GraphLab metric server port: %d' %
                               UnityGlobalProxy(client).get_metric_server_port())
     server.get_logger().info('GraphLab Server Version: %s' %
                              UnityGlobalProxy(client).get_version())
 
+    from graphlab.extensions import _publish
+    _publish()
+
 
 
 # Register an exit callback handler to stop the server on python exit.
