@@ -10,9 +10,10 @@ detailed descriptions and examples, please see the `User Guide
 <http://www.graphlab.com/learn/gallery>`_.
 """
 
-__all__ = ['sframe', 'sarray', 'sgraph', 'sketch']
+__all__ = ['sframe', 'sarray', 'sgraph', 'sketch', 'image']
 
 import sframe
 import sarray
 import sgraph
 import sketch
+import image
