@@ -62,7 +62,7 @@ class SArray(object):
         saved, this is loaded as an SArray read directly out of that
         directory.
 
-    dtype : {None, int, float, str, list, array.array, dict}, optional
+    dtype : {None, int, float, str, list, array.array, dict, datetime.datetime, graphlab.Image}, optional
         The data type of the SArray. If not specified (None), we attempt to
         infer it from the input. If it is a numpy array or a Pandas series, the
         dtype of the array/series is used. If it is a list, the dtype is
@@ -76,6 +76,8 @@ class SArray(object):
     Notes
     -----
     - If ``data`` is pandas.Series, the index will be ignored.
+    - The datetime is based on the Boost datetime format (see http://www.boost.org/doc/libs/1_48_0/doc/html/date_time/date_time_io.html
+      for details)
     - When working with the GraphLab EC2 instance (see
       :py:func:`graphlab.aws.launch_EC2()`), an SArray cannot be constructed
       using local file path, because it involves a potentially large amount of
@@ -89,9 +91,9 @@ class SArray(object):
     >>> sa = SArray('http://s3-us-west-2.amazonaws.com/testdatasets/a_to_z.txt.gz')
     >>> sa = SArray([[1,2,3], [3,4,5]])
     >>> sa = SArray(data=[{'a':1, 'b': 2}, {'b':2, 'c': 1}])
+    >>> sa = SArray(data=[datetime.datetime(2011, 10, 20, 9, 30, 10)])
 
     Working with a GraphLab EC2 instance:
-
     >>> graphlab.aws.launch_EC2('m1.large')
     >>> sa = SArray('~/mydata/foo.csv')          # throws exception
     >>> sa = SArray('remote:///mydata/foo.csv')  # works
@@ -184,7 +186,7 @@ class SArray(object):
 
         >>> graphlab.SArray.from_const(0, 10)
         """
-        assert type(size) is int and size > 0, "size must be a positive int"
+        assert type(size) is int and size >= 0, "size must be a positive int"
         if (type(value) not in set([int, float, str, array.array, list, dict])):
             raise TypeError('Cannot create sarray of value type %s' % str(type(value)))
         proxy = UnitySArrayProxy(glconnect.get_client())
@@ -192,22 +194,80 @@ class SArray(object):
         return cls(_proxy=proxy)
 
     @classmethod
-    def from_avro(cls, url):
+    def from_sequence(cls, *args):
+        """
+        from_sequence(start=0, stop)
+
+        >>> from_sequence(1000)
+        Construct an SArray of integer values from 0 to 999
+
+        This is equivalent, but more efficient than:
+        >>> gl.SArray(range(1000))
+
+        >>> from_sequence(10, 1000)
+        Construct an SArray of integer values from 10 to 999
+
+        This is equivalent, but more efficient than:
+        >>> gl.SArray(range(10, 1000))
+
+        Parameters
+        ----------
+        start : int, optional
+            The start of the sequence. The sequence will contain this value.
+
+        stop : int
+          The end of the sequence. The sequence will not contain this value.
+
+        """
+        start = None
+        stop = None
+        # fill with args. This checks for from_sequence(100), from_sequence(10,100)
+        if len(args) == 1:
+            stop = args[0]
+        elif len(args) == 2:
+            start = args[0]
+            stop = args[1]
+
+        if stop is None and start is None:
+            raise TypeError("from_sequence expects at least 1 argument. got 0")
+        elif start is None:
+            return _create_sequential_sarray(stop)
+        else:
+            size = stop - start
+            # this matches the behavior of range
+            # i.e. range(100,10) just returns an empty array
+            if (size < 0):
+                size = 0
+            return _create_sequential_sarray(size, start)
+
+    @classmethod
+    def from_avro(cls, filename):
         """
         Construct an SArray from an Avro file. The SArray type is determined by
         the schema of the Avro file.
 
         Parameters
         ----------
-        url : str
-          The file to load into an SArray.
+        filename : str
+          The Avro file to load into an SArray.
+
+        Examples
+        --------
+        Construct an SArray from a local Avro file named 'data.avro':
+
+        >>> graphlab.SArray.from_avro('/data/data.avro')
+
+        Notes
+        -----
+        Currently only supports direct loading of files on the local filesystem.
 
         References
         ----------
         - `Avro Specification <http://avro.apache.org/docs/1.7.7/spec.html>`_
         """
+        _mt._get_metric_tracker().track('sarray.from_avro')
         proxy = UnitySArrayProxy(glconnect.get_client())
-        proxy.load_from_avro(url)
+        proxy.load_from_avro(filename)
         return cls(_proxy = proxy)
 
     def __get_content_identifier__(self):
@@ -221,29 +281,47 @@ class SArray(object):
         with cython_context():
             return self.__proxy__.get_content_identifier()
 
-    def save(self, targetfile):
+    def save(self, filename, format=None):
         """
         Saves the SArray to file.
 
-        The saved SArray will be in a directory named with the `targetfile` parameter.
+        The saved SArray will be in a directory named with the `targetfile`
+        parameter.
 
         Parameters
         ----------
-        targetfile: string
-            The directory to save the SArray. Either a local path or a remote
-            URL. If `targetfile` is an existing directory, `save` will fail if
-            the directory is not a GraphLab archive (directory for previously
-            saved GraphLab Create objects), else it will overwrite the existing
-            directory.
+        filename : string
+            A local path or a remote URL.  If format is 'text', it will be
+            saved as a text file. If format is 'binary', a directory will be
+            created at the location which will contain the SArray.
+
+        format : {'binary', 'text', 'csv'}, optional
+            Format in which to save the SFrame. Binary saved SArrays can be
+            loaded much faster and without any format conversion losses.
+            'text' and 'csv' are synonymous: Each SArray row will be written
+            as a single line in an output text file. If not
+            given, will try to infer the format from filename given. If file
+            name ends with 'csv', 'txt' or '.csv.gz', then save as 'csv' format,
+            otherwise save as 'binary' format.
         """
-        with cython_context():
-            self.__proxy__.save(make_internal_url(targetfile))
+        if format == None:
+            if filename.endswith(('.csv', '.csv.gz', 'txt')):
+                format = 'text'
+            else:
+                format = 'binary'
+        if format == 'binary':
+            with cython_context():
+                self.__proxy__.save(make_internal_url(filename))
+        elif format == 'text':
+            sf = gl.SFrame({'X1':self})
+            with cython_context():
+                sf.__proxy__.save_as_csv(make_internal_url(filename), {'header':False})
 
     def __repr__(self):
         """
         Returns a string description of the SArray.
         """
-        ret =       "dtype: " + str(self.dtype().__name__) + "\n"
+        ret = "dtype: " + str(self.dtype().__name__) + "\n"
         ret = ret + "Rows: " + str(self.size()) + "\n"
         ret = ret + str(self)
         return ret
@@ -252,7 +330,12 @@ class SArray(object):
         """
         Returns a string containing the first 100 elements of the array.
         """
-        headln = str(list(self.head(100)))
+
+        # If sarray is image, take head of elements casted to string.
+        if self.dtype() == graphlab.data_structures.image.Image:
+            headln = str(list(self._head_str(100)))
+        else:
+            headln = str(list(self.head(100)))
         if (self.size() > 100):
             # cut the last close bracket
             # and replace it with ...
@@ -539,15 +622,15 @@ class SArray(object):
         Returns
         -------
         out : type
-            The type of the sarray.
+            The type of the SArray.
 
         Examples
         --------
         >>> sa = gl.SArray(["The quick brown fox jumps over the lazy dog."])
-        >>> sa.dtype
+        >>> sa.dtype()
         str
         >>> sa = gl.SArray(range(10))
-        >>> sa.dtype
+        >>> sa.dtype()
         int
         """
         return self.__proxy__.dtype()
@@ -655,36 +738,9 @@ class SArray(object):
         with cython_context():
             return SArray(_proxy=self.__proxy__.vector_slice(start, end))
 
-    def count_words(self, to_lower=True):
+    def _count_words(self, to_lower=True):
         """
-        Count words in the SArray. Return an SArray of dictionary type where
-        each element contains the word count for each word that appeared in the
-        corresponding input element. The words are split on all whitespace and
-        punctuation characters. Only works if this SArray is of string type.
-
-        Parameters
-        ----------
-        to_lower : bool, optional
-            If True, all words are converted to lower case before counting.
-
-        Returns
-        -------
-        out : SArray
-            The SArray of dictionary type, where each element contains the word
-            count for each word that appeared in corresponding input element.
-
-        See Also
-        --------
-        count_ngrams
-
-        Examples
-        --------
-        >>> sa = graphlab.SArray(["The quick brown fox jumps.",
-                                 "Word word WORD, word!!!word"])
-        >>> sa.count_words()
-        dtype: dict
-        Rows: 2
-        [{'quick': 1, 'brown': 1, 'jumps': 1, 'fox': 1, 'the': 1}, {'word': 5}]
+        For documentation, see graphlab.text_analytics.count_ngrams().
         """
         if (self.dtype() != str):
             raise TypeError("Only SArray of string type is supported for counting bag of words")
@@ -699,75 +755,9 @@ class SArray(object):
         with cython_context():
             return SArray(_proxy=self.__proxy__.count_bag_of_words(options))
 
-    def count_ngrams(self, n=2, method="word", to_lower=True, ignore_space=True):
+    def _count_ngrams(self, n=2, method="word", to_lower=True, ignore_space=True):
         """
-        Return an SArray of ``dict`` type where each element contains the count
-        for each of the n-grams that appear in the corresponding input element.
-        The n-grams can be specified to be either character n-grams or word
-        n-grams.  The input SArray must contain strings.
-
-        Parameters
-        ----------
-        n : int, optional
-            The number of words in each n-gram. An ``n`` value of 1 returns word
-            counts.
-
-        method : {'word', 'character'}, optional
-            If "word", the function performs a count of word n-grams. If
-            "character", does a character n-gram count.
-
-        to_lower : bool, optional
-            If True, all words are converted to lower case before counting.
-
-        ignore_space : bool, optional
-            If method is "character", indicates if spaces between words are
-            counted as part of the n-gram. For instance, with the input SArray
-            element of "fun games", if this parameter is set to False one
-            tri-gram would be 'n g'. If ``ignore_space`` is set to True, there
-            would be no such tri-gram (there would still be 'nga'). This
-            parameter has no effect if the method is set to "word".
-
-        Returns
-        -------
-        out : SArray
-            An SArray of dictionary type, where each key is the n-gram string
-            and each value is its count.
-
-        See Also
-        --------
-        count_words
-
-        Notes
-        -----
-        - Ignoring case (with ``to_lower``) involves a full string copy of the
-          SArray data. To increase speed for large documents, set ``to_lower``
-          to False.
-
-        - Punctuation and spaces are both delimiters when counting word n-grams.
-          When counting character n-grams, punctuation is always ignored.
-
-        References
-        ----------
-        - `N-gram wikipedia article <http://en.wikipedia.org/wiki/N-gram>`_
-
-        Examples
-        --------
-        Counting word n-grams:
-
-        >>> from graphlab import SArray
-        >>> sa = SArray(['I like big dogs. I LIKE BIG DOGS.'])
-        >>> sa.count_ngrams(3)
-        dtype: dict
-        Rows: 1
-        [{'big dogs i': 1, 'like big dogs': 2, 'dogs i like': 1, 'i like big': 2}]
-
-        Counting character n-grams:
-
-        >>> sa = SArray(['Fun. Is. Fun'])
-        >>> sa.count_ngrams(3, "character")
-        dtype: dict
-        Rows: 1
-        {'fun': 2, 'nis': 1, 'sfu': 1, 'isf': 1, 'uni': 1}]
+        For documentation, see graphlab.text_analytics.count_ngrams().
         """
         if (self.dtype() != str):
             raise TypeError("Only SArray of string type is supported for counting n-grams")
@@ -1046,7 +1036,7 @@ class SArray(object):
             The function to transform each element. Must return exactly one
             value which can be cast into the type specified by ``dtype``.
 
-        dtype : {None, int, float, str, list, array.array, dict}, optional
+        dtype : {None, int, float, str, list, array.array, dict, graphlab.Image}, optional
             The data type of the new SArray. If ``None``, the first 100 elements
             of the array are used to guess the target data type.
 
@@ -1085,6 +1075,27 @@ class SArray(object):
             import traceback
             if dtype == None:
                 dtype = infer_type_of_list(dryrun)
+            if not seed:
+                seed = time.time()
+
+            # log metric
+            _mt._get_metric_tracker().track('sarray.apply')
+
+            # First phase test if it is a toolkit function
+            nativefn = None
+            try:
+                import graphlab.extensions as extensions
+                nativefn = extensions._build_native_function_call(fn)
+            except:
+                # failure are fine. we just fall out into the next few phases
+                pass
+
+            if nativefn is not None:
+                # this is a toolkit lambda. We can do something about it
+                with cython_context():
+                    return SArray(_proxy=self.__proxy__.transform_native(nativefn, dtype, skip_undefined, seed))
+
+            # Second phase. Try lua compilation if possible
             try:
                 # try compilation
                 if _lua_translate:
@@ -1140,11 +1151,6 @@ class SArray(object):
                 print traceback.format_exc()
                 print "Lua Translation Failed"
 
-        if not seed:
-            seed = time.time()
-
-        _mt._get_metric_tracker().track('sarray.apply')
-
         with cython_context():
             return SArray(_proxy=self.__proxy__.transform(fn, dtype, skip_undefined, seed))
 
@@ -1380,21 +1386,26 @@ class SArray(object):
         with cython_context():
             return self.__proxy__.sum()
 
-
     def mean(self):
         """
-        Mean of all the values in the SArray.
+        Mean of all the values in the SArray, or mean image.
 
         Returns None on an empty SArray. Raises an exception if called on an
-        SArray with non-numeric type.
+        SArray with non-numeric type or non-Image type.
 
         Returns
         -------
-        out : float
-            Mean of all values in SArray
+        out : float | graphlab.Image
+            Mean of all values in SArray, or image holding per-pixel mean
+            across the input SArray.
         """
         with cython_context():
-            return self.__proxy__.mean()
+            if self.dtype() == graphlab.Image:
+                import graphlab.extensions as extensions
+                extensions.list_functions()
+                return extensions.generate_mean(self)
+            else:
+                return self.__proxy__.mean()
 
 
     def std(self, ddof=0):
@@ -1462,6 +1473,145 @@ class SArray(object):
         with cython_context():
             return self.__proxy__.nnz()
 
+    def datetime_to_str(self,str_format="%Y-%m-%dT%H:%M:%S%ZP"):
+        """
+        Create a new SArray with all the values cast to str. The string format is
+        specified by the 'str_format' parameter.
+
+        Parameters
+        ----------
+        str_format : str
+            The format to output the string. Default format is "%Y-%m-%dT%H:%M:%S%ZP".
+
+        Returns
+        -------
+        out : SArray[str]
+            The SArray converted to the type 'str'.
+
+        Examples
+        --------
+        >>> dt = datetime.datetime(2011, 10, 20, 9, 30, 10, tzinfo=GMT(-5))
+        >>> sa = graphlab.SArray([dt])
+        >>> sa.datetime_to_str("%e %b %Y %T %ZP")
+        dtype: str
+        Rows: 1
+        [20 Oct 2011 09:30:10 GMT-05:00]
+
+        See Also
+        ----------
+        str_to_datetime
+
+        References
+        ----------
+        [1] Boost date time from string conversion guide (http://www.boost.org/doc/libs/1_48_0/doc/html/date_time/date_time_io.html)
+
+        """
+        if(self.dtype() != datetime.datetime):
+            raise TypeError("datetime_to_str expects SArray of datetime as input SArray")
+
+        _mt._get_metric_tracker().track('sarray.datetime_to_str')
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.datetime_to_str(str_format))
+
+    def str_to_datetime(self,str_format="%Y-%m-%dT%H:%M:%S%ZP"):
+        """
+        Create a new SArray with all the values cast to datetime. The string format is
+        specified by the 'str_format' parameter.
+
+        Parameters
+        ----------
+        str_format : str
+            The string format of the input SArray. Default format is "%Y-%m-%dT%H:%M:%S%ZP".
+
+        Returns
+        -------
+        out : SArray[datetime.datetime]
+            The SArray converted to the type 'datetime'.
+
+        Examples
+        --------
+        >>> sa = graphlab.SArray(["20-Oct-2011 09:30:10 GMT-05:30"])
+        >>> sa.str_to_datetime("%d-%b-%Y %H:%M:%S %ZP")
+        dtype: datetime
+        Rows: 1
+        datetime.datetime(2011, 10, 20, 9, 30, 10, tzinfo=GMT(-5.5))
+
+        See Also
+        ----------
+        datetime_to_str
+
+        References
+        ----------
+        [1] boost date time to string conversion guide (http://www.boost.org/doc/libs/1_48_0/doc/html/date_time/date_time_io.html)
+
+        """
+        if(self.dtype() != str):
+            raise TypeError("str_to_datetime expects SArray of str as input SArray")
+
+        _mt._get_metric_tracker().track('sarray.str_to_datetime')
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.str_to_datetime(str_format))
+
+    def pixel_array_to_image(self, width, height, channels, undefined_on_failure=True, allow_rounding=False):
+        """
+        Create a new SArray with all the values cast to :py:class:`graphlab.image.Image`
+        of uniform size.
+
+        Parameters
+        ----------
+        width: int
+            The width of the new images.
+
+        height: int
+            The height of the new images.
+
+        channels: int.
+            Number of channels of the new images.
+
+        undefined_on_failure: bool , optional , default True
+            If True, return None type instead of Image type in failure instances.
+            If False, raises error upon failure.
+
+        allow_rounding: bool, optional , default False
+            If True, rounds non-integer values when converting to Image type.
+            If False, raises error upon rounding.
+
+        Returns
+        -------
+        out : SArray[graphlab.Image]
+            The SArray converted to the type 'graphlab.Image'.
+        """
+        if(self.dtype() != array.array):
+            raise TypeError("array_to_img expects SArray of arrays as input SArray")
+
+        num_to_test = 10
+
+        num_test = min(self.size(), num_to_test)
+
+        mod_values = [val % 1 for x in range(num_test) for val in self[x]]
+
+        out_of_range_values = [(val > 255 or val < 0) for x in range(num_test) for val in self[x]]
+
+        if sum(mod_values) != 0.0 and not allow_rounding:
+            raise ValueError("There are non-integer values in the array data. Images only support integer data values between 0 and 255. To permit rounding, set the 'allow_rounding' paramter to 1.")
+
+        if sum(out_of_range_values) != 0:
+            raise ValueError("There are values outside the range of 0 to 255. Images only support integer data values between 0 and 255.")
+
+        _mt._get_metric_tracker().track('sarray.pixel_array_to_img')
+
+        import graphlab.extensions as extensions
+        extensions.list_functions()
+        return extensions.vector_sarray_to_image_sarray(self, width, height, channels, undefined_on_failure)
+
+    def _head_str(self, num_rows):
+        """
+        Takes the head of SArray casted to string.
+        """
+        import graphlab.extensions as extensions
+        extensions.list_functions()
+        return extensions._head_str(self, num_rows)
+
     def astype(self, dtype, undefined_on_failure=False):
         """
         Create a new SArray with all values cast to the given type. Throws an
@@ -1469,7 +1619,7 @@ class SArray(object):
 
         Parameters
         ----------
-        dtype : {int, float, str, list, array.array, dict}
+        dtype : {int, float, str, list, array.array, dict, datetime.datetime}
             The type to cast the elements to in SArray
 
         undefined_on_failure: bool, optional
@@ -1487,6 +1637,9 @@ class SArray(object):
           and list types are quite generic and permit a variety of interesting
           formats to be interpreted. For instance, a JSON string can usually be
           interpreted as a list or a dictionary type. See the examples below.
+        - For datetime-to-string  and string-to-datetime conversions,
+          use sa.datetime_to_str() and sa.str_to_datetime() functions.
+        - For array.array to graphlab.Image conversions, use sa.pixel_array_to_image()
 
         Examples
         --------
@@ -1508,6 +1661,9 @@ class SArray(object):
 
         _mt._get_metric_tracker().track('sarray.astype.%s' % str(dtype.__name__))
 
+        if (dtype == graphlab.Image) and (self.dtype() == array.array):
+            raise TypeError("Cannot cast from image type to array with sarray.astype(). Please use sarray.array_to_img() instead.")
+
         with cython_context():
             return SArray(_proxy=self.__proxy__.astype(dtype, undefined_on_failure))
 
@@ -1735,6 +1891,8 @@ class SArray(object):
             Many of the statistics are approximate.
         """
         from graphlab.data_structures.sketch import Sketch
+        if (self.dtype() == graphlab.data_structures.image.Image):
+            raise TypeError("sketch_summary() is not supported for arrays of image type")
         if (type(background) != bool):
             raise TypeError("'background' parameter has to be a boolean value")
         if (sub_sketch_keys != None):
@@ -1822,13 +1980,30 @@ class SArray(object):
         return SArray(_proxy=res['X1'].__proxy__)
 
     @graphlab.canvas.inspect.find_vars
-    def show(self):
+    def show(self, view=None):
         """
-        show()
+        show(view=None)
         Visualize the SArray with GraphLab Canvas. This function starts Canvas
         if it is not already running. If the SArray has already been plotted,
         this function will update the plot.
 
+        Parameters
+        ----------
+        view : str, optional
+            The name of the SFrame view to show. Can be one of:
+
+            - None: Use the default (depends on the dtype of the SArray).
+            - 'Categorical': Shows most frequent items in this SArray, sorted
+              by frequency. Only valid for str, int, or float dtypes.
+            - 'Numeric': Shows a histogram (distribution of values) for the
+              SArray. Only valid for int or float dtypes.
+            - 'Dictionary': Shows a cross filterable list of keys (categorical)
+              and values (categorical or numeric). Only valid for dict dtype.
+            - 'Array': Shows a Numeric view, filterable by sub-column (index).
+              Only valid for array.array dtype.
+            - 'List': Shows a Categorical view, aggregated across all sub-
+              columns (indices). Only valid for list dtype.
+
         Returns
         -------
         view : graphlab.canvas.view.View
@@ -1843,8 +2018,15 @@ class SArray(object):
         Suppose 'sa' is an SArray, we can view it in GraphLab Canvas using:
 
         >>> sa.show()
+
+        If 'sa' is a numeric (int or float) SArray, we can view it as
+        a categorical variable using:
+
+        >>> sa.show(view='Categorical')
         """
-        return graphlab.canvas.show(graphlab.canvas.views.sarray.SArrayView(self))
+        return graphlab.canvas.show(graphlab.canvas.views.sarray.SArrayView(self, params={
+            'view': view
+        }))
 
     def item_length(self):
         """
@@ -1884,6 +2066,128 @@ class SArray(object):
         with cython_context():
             return SArray(_proxy = self.__proxy__.item_length())
 
+    def split_datetime(self, column_name_prefix = "X", limit=None, tzone=False):
+        """
+        Splits an SArray of datetime type to multiple columns, return a
+        new SFrame that contains expanded columns. A SArray of datetime will be
+        split by default into an SFrame of 6 columns, one for each
+        year/month/day/hour/minute/second element.
+
+        column naming:
+        When splitting a SArray of datetime type, new columns are named:
+        prefix.year, prefix.month, etc. The prefix is set by the parameter
+        "column_name_prefix" and defaults to 'X'. If column_name_prefix is
+        None or empty, then no prefix is used.
+
+        Timezone column:
+        If tzone parameter is True, then timezone information is represented
+        as one additional column which is a float shows the offset from
+        GMT(0.0) or from UTC.
+
+        Parameters
+        ----------
+        column_name_prefix: str, optional
+            If provided, expanded column names would start with the given prefix.
+            Defaults to "X".
+
+        limit: list[str], optional
+            Limits the set of datetime elements to expand.
+            Elements are 'year','month','day','hour','minute',
+            and 'second'.
+
+        tzone: bool, optional
+            A boolean parameter that determines whether to show timezone column or not.
+            Defaults to False.
+
+        Returns
+        -------
+        out : SFrame
+            A new SFrame that contains all expanded columns
+
+        Examples
+        --------
+        To expand only day and year elements of a datetime SArray
+
+         >>> sa = SArray(
+            [datetime(2011, 1, 21, 7, 7, 21, tzinfo=GMT(0)),
+             datetime(2010, 2, 5, 7, 8, 21, tzinfo=GMT(4.5)])
+
+         >>> sa.expand(column_name_prefix=None,limit=['day','year'])
+            Columns:
+                day   int
+                year  int
+            Rows: 2
+            Data:
+            +-------+--------+
+            |  day  |  year  |
+            +-------+--------+
+            |   21  |  2011  |
+            |   5   |  2010  |
+            +-------+--------+
+            [2 rows x 2 columns]
+
+
+        To expand only year and tzone elements of a datetime SArray
+        with tzone column represented as a string. Columns are named with prefix:
+        'Y.column_name'.
+
+        >>> sa.split_datetime(column_name_prefix="Y",limit=['year'],tzone=True)
+            Columns:
+                Y.year  int
+                Y.tzone float
+            Rows: 2
+            Data:
+            +----------+---------+
+            |  Y.year  | Y.tzone |
+            +----------+---------+
+            |    2011  |  0.0    |
+            |    2010  |  4.5    |
+            +----------+---------+
+            [2 rows x 2 columns]
+
+        """
+        if self.dtype() != datetime.datetime:
+            raise TypeError("Only column of datetime type is supported.")
+
+        if column_name_prefix == None:
+            column_name_prefix = ""
+        if type(column_name_prefix) != str:
+            raise TypeError("'column_name_prefix' must be a string")
+
+        # convert limit to column_keys
+        if limit != None:
+            if (not hasattr(limit, '__iter__')):
+                raise TypeError("'limit' must be a list");
+
+            name_types = set([type(i) for i in limit])
+            if (len(name_types) != 1):
+                raise TypeError("'limit' contains values that are different types")
+
+            if (name_types.pop() != str):
+                raise TypeError("'limit' must contain string values.")
+
+            if len(set(limit)) != len(limit):
+                raise ValueError("'limit' contains duplicate values")
+
+        column_types = []
+
+        if(limit != None):
+            column_types = list()
+            for i in limit:
+                column_types.append(int);
+        else:
+            limit = ['year','month','day','hour','minute','second']
+            column_types = [int, int, int, int, int, int]
+
+        if(tzone == True):
+            limit += ['tzone']
+            column_types += [float]
+
+        _mt._get_metric_tracker().track('sarray.split_datetime')
+
+        with cython_context():
+           return gl.SFrame(_proxy=self.__proxy__.expand(column_name_prefix, limit, column_types))
+
     def unpack(self, column_name_prefix = "X", column_types=None, na_value=None, limit=None):
         """
         Convert an SArray of list, array, or dict type to an SFrame with
@@ -2107,8 +2411,8 @@ class SArray(object):
         Rows: 3
         [1, 2, 3]
         """
-        if self.dtype() not in (int, float, str):
-            raise TypeError("Only sarray with type (int, float, str) can be sorted")
+        if self.dtype() not in (int, float, str, datetime.datetime):
+            raise TypeError("Only sarray with type (int, float, str, datetime.datetime) can be sorted")
         sf = gl.SFrame()
         sf['a'] = self
         return sf.sort('a', ascending)['a']
