@@ -163,7 +163,7 @@ class SGraph(object):
     SFrames not used as vertex IDs are assumed to be vertex or edge attributes.
 
     Please see the `User Guide 
-    <http://www.graphlab.com/learn/userguide.html#Working_with_data_Graph_data>`_ 
+    <http://www.graphlab.com/learn/userguide/index.html#Working_with_data_Graph_data>`_ 
     for a more detailed introduction to creating and working with SGraphs.
 
     Parameters
@@ -913,7 +913,19 @@ class SGraph(object):
             missing_fields = set(mutated_fields).difference(set(input_fields))
             raise ValueError('Mutated field \"%s\" does not exist in graph' % str(list(missing_fields)))
 
-        return SGraph(_proxy=g.__proxy__.lambda_triple_apply(triple_apply_fn, mutated_fields))
+        nativefn = None
+        try:
+            import graphlab.extensions as extensions
+            nativefn = extensions._build_native_function_call(triple_apply_fn)
+        except:
+            # failure are fine. we just fall out into the next few phases
+            pass
+        if nativefn is not None:
+            with cython_context():
+                return SGraph(_proxy=g.__proxy__.lambda_triple_apply_native(nativefn, mutated_fields))
+        else:
+            with cython_context():
+                return SGraph(_proxy=g.__proxy__.lambda_triple_apply(triple_apply_fn, mutated_fields))
 
     def save(self, filename, format='auto'):
         """
@@ -1054,7 +1066,7 @@ class SGraph(object):
           displayed as-is. For such graphs, construct a subgraph by selecting
           some vertices and edges, then call this method on the result.
         - See the `user guide
-          <http://www.graphlab.com/learn/userguide.html#Working_with_data_Visualization>`_ for more details and extended examples.
+          <http://www.graphlab.com/learn/userguide/index.html#Working_with_data_Visualization>`_ for more details and extended examples.
 
         Examples
         --------
