@@ -2,7 +2,7 @@ import __builtin__
 import collections as _collections
 import tempfile as _tempfile
 import shutil as _shutil
-import imp
+import imp as _imp
 import copy as _copy
 import os as _os
 import fnmatch as _fnmatch
@@ -471,9 +471,11 @@ class Task(object):
                 if directory is None:
                     directory = _os.path.join(_os.path.dirname(_os.path.abspath(code)))
 
-                # TODO: confirm that function exists in file provided
-                # use getattr and try to load the function in the module
-                # if fails then fail this call here
+                # confirm that function exists in file provided use getattr and try to load the function in the module
+                module = _imp.load_source('', code)
+                if not hasattr(module, function_name):
+                    raise TypeError("Unable to find function name \"%s\" from the code file spcified, please verify the function name and the code file. Code file searched: %s" % (function_name, code))
+                self._data['codestr'] = _inspect.getsource(getattr(module, function_name))
 
                 files = {}
                 for f in Task.__find_files(directory, pattern):
@@ -1010,14 +1012,29 @@ class Task(object):
 
         return out
 
+    def __code_snapshot__(self):
+        if 'codestr' in self._data and self._data['codestr'] is not None:
+            return self._data['codestr']
+        return self.__code_str__()
+            
     def __to_dict__(self):
+        params = self.get_params()
+        param_dict = {}
+        isFunction = False
+        for k,v in params.iteritems():
+            param_dict[k] = v
+            if _isfunction(v):
+                param_dict[k] = v.func_name
+                isFunction = True
+            
         task_json = {}
         task_json["name"] = self.get_name()
         task_json["description"] = self.get_description()
-        task_json["inputs"] = self.__inputs_str__()
-        task_json["code"] = self.__code_str__()
+        task_json["inputs"] = self.get_inputs()
+        task_json["code"] = self.__code_snapshot__()
         task_json["outputs"] = self.get_outputs()
-        task_json["params"] = self.get_params()
+        task_json["params"] = param_dict
+        task_json["isFunction"] = isFunction
         task_json["required_packages"] = list(self.get_required_packages())
         return task_json
 
