@@ -88,9 +88,6 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
             job._end_time = str(_datetime.datetime.now().isoformat())
             raise e
 
-        if session is not None:
-            session.save(job)
-
         return job
 
     def run_task(self, task, metrics, location=None, status_path=None):
@@ -110,8 +107,9 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
         __LOGGER__.info("Task started: %s" % task.get_name())
         # update task status, for both local and async if possible
         task.status = 'Running'
+        task._starttime = starttime
         if status_path is not None:
-            self.__write_task_status(task, status_path)
+            self.__write_task_status(task, status_path, starttime=starttime)
 
         # path for execution
         if location is None:
@@ -170,8 +168,9 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
             metrics['task.%s.metrics' % task.name] = runtime_task.metrics
             # update task status when task is completed
             task.status = 'Completed'
+            task._runtime = str(t.secs)
             if status_path is not None:
-                self.__write_task_status(task, status_path)
+                self.__write_task_status(task, status_path, runtime=str(t.secs))
 
         finally:
             if path in _sys.path:
@@ -180,11 +179,15 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
 
         return runtime_task
 
-    def __write_task_status(self, task, status_path):
+    def __write_task_status(self, task, status_path, starttime=None, runtime=None):
         if status_path is None:
             return
         with open(status_path, 'a+') as f:
             f.writelines(str(task.name + ":" + task.status + "\n"))
+            if starttime is not None:
+                f.writelines(str(task.name + "_starttime:" + starttime + "\n"))
+            if runtime is not None:
+                f.writelines(str(task.name + "_runtime:" + runtime + "\n"))
             f.close()
 
     def _run(self, job, session=None, status_path=None):
@@ -205,15 +208,12 @@ class LocalExecutionEnvironment(ExecutionEnvironment):
                 task.env = self
                 self.run_task(task, job.metrics, location=exec_dir, status_path=status_path)
                 if not isinstance(task, _job.Job): # update task status in job if task is not job
+                    job._task_status[task.name+"_starttime"] = task._starttime
+                    job._task_status[task.name+"_runtime"] = task._runtime
                     job._task_status[task.name] = task.status
-                if session is not None:
-                    session.save(job)
                 self._completed[task.name] = task
 
         __LOGGER__.info("Execution completed: %s" % job.name)
-    
-        if session is not None:
-            session.save(job, typename=str(_job.Job.__name__))
         return job
 
     def get_dependent_task(self, name):
@@ -250,7 +250,8 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
         user_data = {
             'daemon': True, 'num_children_host': environment.num_hosts - 1,
             'aws_access_key': credentials['aws_access_key_id'],
-            'aws_secret_key': credentials['aws_secret_access_key']
+            'aws_secret_key': credentials['aws_secret_access_key'],
+            'auth_token': ''
             }
 
         if('GRAPHLAB_TEST_AMI_ID' in _os.environ and 'GRAPHLAB_TEST_ENGINE_URL' in _os.environ):
@@ -262,7 +263,7 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
         # Launch the (commander) EC2 host
         commander = _ec2_factory(environment.instance_type, region = environment.region,
                                  CIDR_rule = environment.CIDR_rule,
-                                 security_group = environment.security_group,
+                                 security_group_name = environment.security_group,
                                  tags = environment.tags, user_data = user_data,
                                  credentials = credentials)
         return commander
@@ -280,6 +281,7 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
         s3_log_folder = job.environment.s3_log_folder_path + job.name + '-' + str(_time.time()) + '/'
         s3_state_key = s3_log_folder + 'status'
         s3_log_key = s3_log_folder + 'log.txt'
+        s3_status_key = s3_log_folder + 'jobstatus.txt'
 
         # Make sure we can write to S3
         try:
@@ -309,6 +311,7 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
         post_params = {'s3_bucket': job.environment.s3_bucket,
                   's3_state_key': s3_state_key, 
                   's3_log_key': s3_log_key,
+                  's3_status_key': s3_status_key,
                   'required_packages': job.get_required_packages()
                   }
         while current_tries <  MAX_TRIES:
@@ -344,7 +347,7 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
 
         result = _job.Ec2Job(job.name, job.tasks, job.environment, commander.public_dns_name,
                              commander.instance_id, job.environment.s3_bucket, s3_state_key,
-                             s3_log_key)
+                             s3_log_key, s3_status_key)
 
         if session is not None:
             session.save(result)
@@ -384,7 +387,7 @@ class HadoopExecutionEnvironment(ExecutionEnvironment):
 
             # format hadoop command
             if job.environment.get_config_dir() is not None:
-                hadoop_base = "hadoop --config %s " % job.environment.get_config_dir()
+                hadoop_base = "hadoop --config %s " % _os.path.expanduser(job.environment.get_config_dir())
             else:
                 hadoop_base = "hadoop "
 
@@ -487,6 +490,11 @@ class LocalAsynchronousEnvironment(ExecutionEnvironment):
         3. Start additional process
         4. Return LocalAsynchronousJob object to caller
         """
+        # log if job has required packages, since not installing them
+        required_packages = job.get_required_packages()
+        if required_packages:
+            __LOGGER__.info("Required packages are not being installed since running locally.")
+
         # process launch with a pickled Job as file path and session.location
         driver_file_path = _os.path.join(_os.path.dirname(__file__), '_graphlabJob.py')
         path = _tempfile.mkdtemp(prefix='graphlab-async-')
