@@ -1,6 +1,7 @@
 import graphlab_util.cloudpickle as _cloudpickle
 import graphlab as _gl
 import _artifact
+import graphlab.canvas as _canvas
 
 import boto.ec2 as _boto
 from boto import connect_s3 as _connect_s3
@@ -12,12 +13,13 @@ import pickle as _pickle
 import pprint as _pprint
 import tempfile as _tempfile
 import zipfile as _zipfile
-import psutil as _psutil
+import graphlab_psutil as _psutil
 import requests as _requests
 import os as _os
 import shutil as _shutil
 import copy as _copy
 import subprocess as _subprocess
+import datetime as _datetime
 import json
 
 import logging
@@ -114,7 +116,9 @@ class Job(object):
         -------
         String representation of the start time
         """
-        return self._start_time
+        if hasattr(self, "_start_time"):
+            return self._start_time
+        return None
 
     def _get_end_time(self):
         """
@@ -124,7 +128,9 @@ class Job(object):
         -------
         String representation of the start time
         """
-        return self._end_time
+        if hasattr(self, "_end_time"):
+            return self._end_time
+        return None
    
     def _get_task_status(self):
         """
@@ -134,8 +140,22 @@ class Job(object):
         -------
         A dictionary representation of the tasks status
         """
-        return self._task_status
+        if hasattr(self, "_task_status"):
+            return self._task_status
+        return None
     
+    def _get_log_file_path(self):
+        """
+        Returns the path to the execution log file of the job
+
+        Returns
+        -------
+        A string representation of execution log file path
+        """
+        if hasattr(self, 'log_file_path'):
+            return self.log_file_path
+        return None
+
     def cancel(self):
         """
         Cancels this execution.
@@ -245,6 +265,25 @@ class Job(object):
         self._session.save(self, typename=self._typename)
         __LOGGER__.info("Job saved successfully.")
 
+    @_canvas.inspect.find_vars
+    def show(self):
+        """
+        show()
+        Visualize the Job with GraphLab Canvas. This function starts Canvas
+        if it is not already running.
+        
+        Returns
+        -------
+        view: graphlab.canvas.view.View
+            An object representing the GraphLab Canvas view
+
+        See Also
+        --------
+        canvas
+        """
+        _canvas.get_target().state.set_selected_variable(('Jobs', self.name))
+        return _canvas.show()
+
     @staticmethod
     def _get_task_names(tasks):
         names = []
@@ -308,7 +347,7 @@ class LocalAsynchronousJob(Job):
         status = self.get_status()
 
         if status == 'Completed' and not hasattr(self, 'metrics'):
-            self.__get_metrics()
+            self._get_metrics()
 
         result = self._get_str()
         result += "Status: %s\n\n" % status
@@ -342,22 +381,22 @@ class LocalAsynchronousJob(Job):
             else:
                 status = str(proc.status).capitalize()
             if status in ['Zombie']:
-                return self.__get_status()
+                return self._get_status()
             elif status in ['Sleeping', 'Disk-sleep']:
                 return 'Running'
             else:
                 return status
         else:
-            return self.__get_status()
+            return self._get_status()
     
-    def get_start_time(self):
+    def _get_start_time(self):
         if '__starttime__' not in self._task_status:
-            self.__get_task_status()
+            self._get_task_status_from_file()
         if '__starttime__' in self._task_status:
             return self._task_status["__starttime__"]
         return None
 
-    def get_end_time(self):
+    def _get_end_time(self):
         if '__endtime__' in self._task_status:
             return self._task_status["__endtime__"]
         return None
@@ -368,14 +407,14 @@ class LocalAsynchronousJob(Job):
             return getattr(self, 'metrics')
         return {}
 
-    def __get_task_status(self):
+    def _get_task_status_from_file(self):
         task_list = {}
         try:
             with open(_os.path.join(self.exec_dir, 'status'), 'r') as f:
                 for line in f:
                     line_split = line.split(":");
-                    if (len(line_split) == 2):
-                        task_list[line_split[0]] = line_split[1].rstrip('\n')
+                    if (len(line_split) == 2 or len(line_split) == 4):
+                        task_list[line_split[0]] = ":".join(line_split[1:]).rstrip('\n')
                     else:
                         if '__starttime__' in task_list:
                             task_list["__endtime__"] = line.rstrip('\n')
@@ -387,13 +426,13 @@ class LocalAsynchronousJob(Job):
         self._task_status = task_list
         return task_list
 
-    def get_task_status(self):
+    def _get_task_status(self):
         # pull from task status file until the job finishes
         if self._finalized is False:
-            self.__get_task_status()
+            self._get_task_status_from_file()
         return self._task_status
 
-    def __get_metrics(self):
+    def _get_metrics(self):
         """ Go to executed job and pull the metrics out
         """
         try:
@@ -406,7 +445,7 @@ class LocalAsynchronousJob(Job):
         import ast
         self.metrics = ast.literal_eval(metrics)
 
-    def __get_status(self):
+    def _get_status(self):
         # get exit code from dir
         try:
             with(open(_os.path.join(self.exec_dir, 'exitcode'), 'r')) as f:
@@ -428,13 +467,13 @@ class LocalAsynchronousJob(Job):
     def __finalize_save(self):
         # when the job finishes, query the metrics and task status one last time, and save
         self._finalized = True
-        self.__get_metrics()
-        self.__get_task_status()
+        self._get_metrics()
+        self._get_task_status_from_file()
         self.save()
 
 class Ec2Job(Job):
     def __init__(self, name, tasks, environment, ec2_dns_name, ec2_instance_id, s3_bucket,
-                 s3_state_key, s3_log_key):
+                 s3_state_key, s3_log_key, s3_status_key):
         super(Ec2Job, self).__init__(name, tasks=tasks, environment=environment)
 
         self.ec2_dns_name = ec2_dns_name
@@ -442,7 +481,12 @@ class Ec2Job(Job):
         self.s3_bucket = s3_bucket
         self.s3_state_key = s3_state_key
         self.s3_log_key = s3_log_key
-
+        self.s3_status_key = s3_status_key
+        self.__s3_log_url_expire = None
+        self.__s3_log_url = None
+        self.__s3_log_size = None
+        self._start_time = None
+        self._end_time = None
         self._final_status = None
 
     def get_status(self):
@@ -461,16 +505,119 @@ class Ec2Job(Job):
 
         # The key only gets deleted on success.
         if not k:
+            self._get_task_status_from_s3()
             self._final_status = 'Completed'
+            self.save()
             return self._final_status
 
         status_from_s3 = k.read()
         if status_from_s3 == 'Failed':
+            self._get_task_status_from_s3()
             self._final_status = 'Failed'
+            self.save()
             return self._final_status
 
         return status_from_s3
 
+    def _get_start_time(self):
+        if self._start_time:
+            return str(self._start_time.isoformat())
+
+        # get starttime from job status
+        self._get_task_status_from_s3()
+        cur_starttime = _datetime.datetime.now()
+        if self._task_status and len(self._task_status) > 0:
+            for key in self._task_status:
+                if key.endswith("starttime"):
+                    starttime_str = self._task_status[key]
+                    starttime = _datetime.datetime.strptime(starttime_str, "%Y-%m-%d %H:%M:%S.%f")
+                    if cur_starttime > starttime:
+                        cur_starttime = starttime
+            self._start_time = cur_starttime
+            return str(self._start_time.isoformat())
+        else:
+            return None
+
+    def _get_task_status_from_s3(self):
+        task_list = {}
+        
+        # Check s3 for the status key.
+        credentials = {}   # If this is empty, boto will look in environment variables.
+        if(hasattr(self.environment, 'aws_access_key') and hasattr(self.environment, 'aws_secret_key')):
+            credentials['aws_access_key_id'] = self.environment.aws_access_key
+            credentials['aws_secret_access_key'] = self.environment.aws_secret_key
+        s3_conn = _connect_s3(**credentials)
+        bucket = s3_conn.get_bucket(self.s3_bucket)
+        k = bucket.get_key(self.s3_status_key)
+
+        try:
+            if k:
+                jobstatus = k.read()
+                jobstatus_blob = jobstatus.split('\n')
+                for line in jobstatus_blob:
+                    line_split = line.split(":");
+                    # spliting the line read, to key value pairs
+                    if (len(line_split) > 1):
+                        task_list[line_split[0]] = ":".join(line_split[1:]).rstrip('\n')
+        except IOError:
+            __LOGGER__.warning("Unable to get job status file from s3://%s/%s" % (self.s3_bucket, self.s3_status_key))
+
+        self._task_status = task_list
+        return task_list
+
+    def _get_task_status(self):
+        # pull from task status file until the job finishes
+        if not self._final_status or not self._task_status:
+            self._get_task_status_from_s3()
+        return self._task_status
+
+    def _get_end_time(self):
+        if self._end_time:
+            return str(self._end_time.isoformat())
+
+        # return None if job has not completed
+        if not self._final_status:
+            return None
+            
+        # get endtime from job status
+        self._get_task_status_from_s3()
+        if self._task_status and len(self._task_status) > 0:
+            self._get_start_time()
+            starttime = self._start_time
+            total_time_sec = 0
+            for key in self._task_status:
+                if key.endswith("runtime"):
+                    duration = int(float(self._task_status[key]))
+                    total_time_sec += duration
+            self._end_time = starttime + _datetime.timedelta(0, total_time_sec)
+            return str(self._end_time.isoformat())
+        else:
+            return None
+
+    def _get_log_file_path(self):
+        # return generated url if the url has not expire
+        if self.__s3_log_url_expire and self.__s3_log_url:
+            delta = _datetime.datetime.now() - self.__s3_log_url_expire
+            if delta.total_seconds() < 300:
+                return (self.__s3_log_url, self.__s3_log_size)
+
+        # Check s3 for the status key.
+        credentials = {}   # If this is empty, boto will look in environment variables.
+        if(hasattr(self.environment, 'aws_access_key') and hasattr(self.environment, 'aws_secret_key')):
+            credentials['aws_access_key_id'] = self.environment.aws_access_key
+            credentials['aws_secret_access_key'] = self.environment.aws_secret_key
+        s3_conn = _connect_s3(**credentials)
+        bucket = s3_conn.get_bucket(self.s3_bucket)
+        k = bucket.get_key(self.s3_log_key)
+
+        if not k:
+            return "Log file not available on S3."
+        else:
+            url = k.generate_url(expires_in=300)
+            self.__s3_log_url = url
+            self.__s3_log_size = k.size
+            self.__s3_log_url_expire = _datetime.datetime.now()
+            return (url, k.size)
 
     def cancel(self):
         cancel_url = "http://%s:9004/cancel" % self.ec2_dns_name
