@@ -1,6 +1,6 @@
 import __builtin__
 import os as _os
-import pickle as _pickle
+import cPickle as _pickle
 import re as _re
 import fnmatch as _fnmatch
 import tempfile as _tempfile
@@ -13,6 +13,8 @@ import environment as _environment
 import _artifact as _artifact
 import _executionenvironment as _env
 import _job as _job
+import _predictive_service._predictive_service as _predictive_service
+import _predictive_service._predictive_service_endpoint as _predictive_service_endpoint
 
 import graphlab as _gl
 
@@ -24,7 +26,7 @@ class Session(object):
     """
     Manages Tasks, Environments, and Jobs.
 
-    Sessions provide functionality for managing the lifecycle of: Tasks, 
+    Sessions provide functionality for managing the lifecycle of: Tasks,
     Environments, and Jobs. Session is not intended to be interacted with
     directly, instead use graphlab.deploy.tasks, graphlab.deploy.jobs,
     graphlab.deploy.environments.
@@ -82,21 +84,22 @@ class Session(object):
 
         if artifact.name in self._objects or artifact.name in self._cache:
             raise NameError("An item with the name '%s' already exists, please rename." % artifact.name)
-
         self._objects[artifact.name] = artifact
+
         return True
 
     def list(self, refresh=False, typename=None):
         """
-        Lists all items managed by this session. Items include Tasks,
-        Environments, and Jobs.
+        List all items managed by this session. Items include Tasks,
+        Environments, Jobs and Predictive Services.
 
         Parameters
         ----------
         refresh : boolean
             Refresh the listing, meaning re-query location to get current artifacts.
         typename : str, optional
-            Optionally limit the output a specific type (Task, Environment, Job)
+            Optionally limit the output a specific type (Task, Environment, Job,
+            PredictiveService)
 
         Returns
         -------
@@ -131,6 +134,9 @@ class Session(object):
                 required_type = _environment.Environment
             elif typename == 'Task':
                 required_type = _artifact.Task
+            elif typename == 'PredictiveService':
+                required_type = (_predictive_service_endpoint.PredictiveServiceEndpoint,
+                                 _predictive_service.PredictiveService)
             else:
                 raise Exception("Unknown typename: %s" % typename)
 
@@ -142,15 +148,82 @@ class Session(object):
         else:
             filtered_values = self._cache.values()
 
-        names = _gl.SArray(data=[n for (n, t, status) in filtered_values])
-        types = _gl.SArray(data=[str(t.__name__) for (n, t, status) in filtered_values])
-        statuses = _gl.SArray(data=[str(status) for (n, t, status) in filtered_values])
+        names = _gl.SArray(data=[n for (n, t, status) in filtered_values], dtype=str)
+        types = _gl.SArray(data=[str(t.__name__) for (n, t, status) in filtered_values], dtype=str)
+        statuses = _gl.SArray(data=[str(status) for (n, t, status) in filtered_values], dtype=str)
 
         sf = _gl.SFrame()
         sf['Name'] = names
         sf['Type'] = types
         sf['Unsaved changes?'] = statuses
-        sf.add_row_number(column_name='Index', start=0)
+        sf = sf.add_row_number(column_name='Index', start=0)
+
+        return sf
+
+    def _list(self, typename=None):
+        """
+        List all items' filenames managed by this session with specific type.
+        Items include Tasks, Environments, Jobs and Predictive Services.
+
+        Parameters
+        ----------
+        typename : str, optional
+            Optionally limit the output a specific type (Task, Environment, Job)
+
+        Returns
+        -------
+        An Array with file names
+        """
+        if typename == 'PredictiveService':
+            return self.__list_ps()
+
+        # Query session dir everytime, but only updates ones not in cache
+        artifact_list = []
+        files = _os.listdir(self.location)
+        for f in files:
+            try:
+                (filename, ext) = _os.path.splitext(f)
+                if ext is None or (typename and ext[1:] != typename):
+                    continue
+                artifact_list.append(filename)
+            except Exception as e:
+                __LOGGER__.warning('Error adding file: %s to artifact list. error: %s' % (f, e))
+
+        return artifact_list
+
+    def __list_ps(self):
+        """
+        List all the Predictive Service endpoints managed by this session.
+
+        Returns
+        -------
+        SFrame of Predictive Services endpoints: S3 path, name and In-Memory?
+        """
+        typename = 'PredictiveService'
+        ps_catalog = {}
+        files = _os.listdir(self.location)
+        for f in files:
+            try:
+                (filename, ext) = _os.path.splitext(f)
+                if ext is None or (typename and ext[1:] != typename):
+                    continue
+                self._open(filename, ps_catalog, typename=typename)
+            except Exception as e:
+                __LOGGER__.warning('Error adding file: %s to artifact list. error: %s' % (f, e))
+
+        ps_info = {}
+        for ps in ps_catalog.itervalues():
+            ps_info[ps.name] = (ps.name, ps.s3_path, 'Yes' if ps.name in self._objects else 'No')
+
+        s3_paths = _gl.SArray(data=[s3_path for (name, s3_path, status) in ps_info.values()])
+        names = _gl.SArray(data=[name for (name, s3_path, status) in ps_info.values()])
+        statuses = _gl.SArray(data=[status for (name, s3_path, status) in ps_info.values()])
+
+        sf = _gl.SFrame()
+        sf['Name'] = names
+        sf['S3_path'] = s3_paths
+        sf['In-Memory?'] = statuses
+        sf = sf.add_row_number(column_name='Index', start=0)
 
         return sf
 
@@ -194,6 +267,8 @@ class Session(object):
                             required_type = _environment.Environment
                         elif typename == 'Task':
                             required_type = _artifact.Task
+                        elif typename == 'PredictiveService':
+                            required_type = _predictive_service.PredictiveService
                         else:
                             __LOGGER__.warning("Unknown typename of file: %s" % typename)
 
@@ -202,7 +277,11 @@ class Session(object):
 
                     __LOGGER__.info("Deleting file: %s" % file)
                     artifact = self.__open_artifact(_os.path.join(loaddir, file), {})
-                    self.__remove_artifact(artifact.name, typename=typename)
+                    if artifact:
+                        self.__remove_artifact(artifact.name, typename=typename)
+                    else:
+                        # invalidate cache if we couldn't remove a file, not sure if it exists or not
+                        self._cache = {}
                 except Exception as e:
                     __LOGGER__.warning("Error in removing, deleted file instead: %s, error: %s" % (file, e))
                     # invalidate cache since error deleting, go to disk for future
@@ -229,6 +308,8 @@ class Session(object):
             required_type = _environment.Environment
         elif typename == 'Task':
             required_type = _artifact.Task
+        elif typename == 'PredictiveService':
+            required_type = _predictive_service.PredictiveService
         else:
             raise TypeError("Invalid typename parameter")
 
@@ -289,6 +370,13 @@ class Session(object):
                 return obj
             elif isinstance(obj, _artifact.Task) and typename == 'Task':
                 return obj
+            elif isinstance(obj, _predictive_service.PredictiveService) and typename == 'PredictiveService':
+                return obj
+
+        # load predictive service from s3 path
+        if typename == 'PredictiveService':
+            endpoint = self._open(name, self._objects, typename=typename)
+            return self.__open_ps(endpoint)
 
         return self._open(name, self._objects, typename=typename)
 
@@ -320,14 +408,19 @@ class Session(object):
                 typename = 'Environment'
             elif isinstance(obj, _artifact.Task):
                 typename = 'Task'
+            elif isinstance(obj, _predictive_service.PredictiveService):
+                typename = 'PredictiveService'
             else:
                 __LOGGER__.error("Trying to save an unrecognized item of type: %s, saving failed." % type(obj))
                 return
 
+        # overwrite the obj with PredictiveServiceEndpoint to be pickled and saved
+        if isinstance(obj, _predictive_service.PredictiveService) and typename == 'PredictiveService':
+            obj = _predictive_service_endpoint.PredictiveServiceEndpoint(obj.name, obj._s3_state_path, obj.aws_credentials)
+
         filename = self._get_filename_from_name(obj.name) + "." + typename
 
         try:
-
             with __builtin__.open(str(_os.path.join(savedir, filename)), "w") as f:
                 _cloudpickle.dump(obj, f)
 
@@ -373,7 +466,9 @@ class Session(object):
         Helper method that returns True when type is supported by Session.
         """
         return any([isinstance(obj, _artifact.Task), isinstance(obj, _artifact.Pipeline),
-            isinstance(obj, _environment.Environment), isinstance(obj, _job.Job)])
+            isinstance(obj, _environment.Environment), isinstance(obj, _job.Job),
+            isinstance(obj, _predictive_service.PredictiveService),
+            isinstance(obj, _predictive_service_endpoint.PredictiveServiceEndpoint)])
 
     def __repr__(self):
         return self.__str__(typename=None)
@@ -415,10 +510,34 @@ class Session(object):
             return self.__open_artifact(path, catalog)
         return None
 
+    def __open_ps(self, endpoint):
+        """
+        Instantiate the Predictive Service from the s3 path. Store a reference of
+        the instance in the current session.
+        """
+        if endpoint is None:
+            return
+
+        s3_path = endpoint.s3_path
+        aws_credentials = endpoint.aws_credentials
+        aws_access_key_id = aws_credentials['aws_access_key_id']
+        aws_secret_access_key = aws_credentials['aws_secret_access_key']
+
+        # loads from the s3 path
+        ps = _gl.deploy.predictive_service.load(s3_path, aws_access_key_id, aws_secret_access_key, override=True)
+        ps._session = self
+        return ps
+
+
     def __open_artifact(self, path, catalog):
         try:
             with __builtin__.open(path, 'r') as f:
-                artifact = _pickle.load(f)
+                artifact = None
+                try:
+                    artifact = _pickle.load(f)
+                except Exception, e:
+                    __LOGGER__.debug("Exception when trying to unpickle: %s" % e)
+                    return None
                 if not Session._is_known_type(artifact):
                     raise Exception("Trying to load an unknown type")
                 artifact._session = self
@@ -470,7 +589,7 @@ class ScopedSession(object):
 
     def list(self, refresh=True):
         """
-        Lists all items of this type (jobs, tasks, environments).
+        List all items of this type (Job, Task, Environment, Predictive Service).
 
         Parameters
         ----------
@@ -483,29 +602,41 @@ class ScopedSession(object):
         """
         return self._session.list(refresh=refresh, typename=self._typename)
 
-    def load(self, item):
+    def _list(self):
         """
-        Loads an item by name.
-
-        Parameters
-        ----------
-        name : str
-            Name of item to load, full name required (ex. s0)
+        List all items of this type (Job, Task, Environment, Predictive Service).
 
         Returns
         -------
-        Instance of the item.
+        An array with columns for name, type
         """
-        return self._session._load(item, typename=self._typename)
+        return self._session._list(typename=self._typename)
 
-    def delete(self, item, pattern=None):
+    def show(self):
         """
-        Deletes an item from disk.        
+        Show all items of this type (Job, Predictive Service) in Canvas 
+        for visualization.
+        """
+        if self._typename == 'Job':
+            import graphlab.canvas
+            import graphlab.canvas.views.job_session
+            return graphlab.canvas.show(graphlab.canvas.views.job_session.ScopedSessionView(self))
+        elif self._typename == 'PredictiveService':
+            import graphlab.canvas
+            import graphlab.canvas.views.ps_session
+            return graphlab.canvas.show(graphlab.canvas.views.ps_session.PredictiveServiceList(self))
+        return 'Canvas visualization is not supported for typename %s.' % self._typename
+
+    def delete(self, item=None, pattern=None):
+        """
+        Delete an item from local disk. For Predictive Service, this operation would
+        only delete the locally saved endpoint to the Predictive Service, and 
+        will NOT terminate the Predictive Service.
 
         Parameters
         ----------
         item : object | str
-            Item to delete, passed as an object or by name as a string (Job, Task, Environment)
+            Item to delete, passed as an object or by name as a string (Job, Task, Environment, Predictive Service)
         pattern : str
             Spec to delete artifacts ex. "*-Local-*". This uses fnmatch.fnmatch
 
