@@ -49,7 +49,15 @@ def is_product_key_valid(product_key, config=DEFAULT_CONFIG):
 
     @param key to validate
     @return True if validates correctly, False otherwise
+
+    Raises a RuntimeError if the unity_server binary cannot be executed.
     """
+    try:
+        cmd = "%s --help" % (config.server_bin)
+        subprocess.check_call(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, shell=True)
+    except:
+        raise RuntimeError("Cannot execute unity_server binary")
+
     try:
         cmd = "%s --check_product_key_only --product_key='%s'" % (config.server_bin, product_key)
         subprocess.check_output(cmd, shell=True)
