@@ -1,6 +1,7 @@
+import graphlab.connect as _mt
 from graphlab.data_structures.sgraph import SGraph as _SGraph
-import graphlab.toolkits.main as _main
-from graphlab.toolkits.graph_analytics.model_base import GraphAnalyticsModel as _ModelBase
+import graphlab.toolkits._main as _main
+from graphlab.toolkits.graph_analytics._model_base import GraphAnalyticsModel as _ModelBase
 
 
 def get_default_options():
@@ -11,46 +12,53 @@ def get_default_options():
     -------
     out : dict
 
+    See Also
+    --------
+    PagerankModel.get_current_options
+
     Examples
     --------
     >>> graphlab.pagerank.get_default_options()
     """
+    _mt._get_metric_tracker().track('toolkit.graph_analytics.pagerank.get_default_options')
+
     return _main.run('pagerank_default_options', {})
 
 
 class PagerankModel(_ModelBase):
     """
-    Model object containing the pagerank value for each vertex.
+    A PageRankModel object contains the pagerank value for each vertex.
     The pagerank value characterize the importance of a vertex
     in the graph using the following recursive definition:
 
         .. math::
-          pr(i) =  RESET_PROB + (1-RESET_PROB) \sum_{j\in N(i)} pr(j) / out_degree(j)
+          pr(i) =  reset_probability + (1-reset_probability) \sum_{j\in N(i)} pr(j) / out_degree(j)
 
     where :math:`N(i)` is the set containing all vertices :math:`j` such that
     there is an edge going from :math:`j` to :math:`i`.
 
     Below is a list of queryable fields for this model:
 
-    +----------------+-----------------------------------------------------------+
-    | Field          | Description                                               |
-    +================+===========================================================+
-    | reset_prob     | The probablity of randomly jumps to any node in the graph |
-    +----------------+-----------------------------------------------------------+
-    | graph          | A new SGraph with the pagerank as a vertex property       |
-    +----------------+-----------------------------------------------------------+
-    | delta          | Change in pagerank for the last iteration in L1 norm      |
-    +----------------+-----------------------------------------------------------+
-    | pagerank       | An SFrame with each vertex's pagerank                     |
-    +----------------+-----------------------------------------------------------+
-    | num_iterations | Number of iterations                                      |
-    +----------------+-----------------------------------------------------------+
-    | threshold      | The convergence threshold in L1 norm                      |
-    +----------------+-----------------------------------------------------------+
-    | runtime        | Total runtime of the toolkit                              |
-    +----------------+-----------------------------------------------------------+
-    | max_iterations | The maximun number of iterations to run                   |
-    +----------------+-----------------------------------------------------------+
+    +-------------------+-----------------------------------------------------------+
+    | Field             | Description                                               |
+    +===================+===========================================================+
+    | reset_probability | The probablity of random jumps to any node in the graph   |
+    +-------------------+-----------------------------------------------------------+
+    | graph             | A new SGraph with the pagerank as a vertex property       |
+    +-------------------+-----------------------------------------------------------+
+    | delta             | Total changes in pagerank during the last iteration       |
+    |                   | (the L1 norm of the changes)                              |
+    +-------------------+-----------------------------------------------------------+
+    | pagerank          | An SFrame with each vertex's pagerank                     |
+    +-------------------+-----------------------------------------------------------+
+    | num_iterations    | Number of iterations                                      |
+    +-------------------+-----------------------------------------------------------+
+    | threshold         | The convergence threshold in L1 norm                      |
+    +-------------------+-----------------------------------------------------------+
+    | training_time     | Total training time of the model                          |
+    +-------------------+-----------------------------------------------------------+
+    | max_iterations    | The maximun number of iterations to run                   |
+    +-------------------+-----------------------------------------------------------+
 
     An instance of this model can be created using
     :func:`graphlab.pagerank.create`.  Do NOT construct the model directly.
@@ -76,15 +84,15 @@ class PagerankModel(_ModelBase):
 
     def _setting_fields(self):
         ret = super(PagerankModel, self)._setting_fields()
-        for k in ['reset_prob', 'threshold', 'max_iterations']:
+        for k in ['reset_probability', 'threshold', 'max_iterations']:
             ret[k] = self[k]
         return ret
 
 
-def create(graph, reset_prob=0.15,
+def create(graph, reset_probability=0.15,
            threshold=1e-2,
            max_iterations=20,
-           verbose=True, plot=False):
+           verbose=True):
     """
     Compute the PageRank for each vertex in the graph. Return a model object
     with total PageRank as well as the PageRank value for each vertex in the
@@ -95,7 +103,7 @@ def create(graph, reset_prob=0.15,
     graph : SGraph
         The graph on which to compute the pagerank value.
 
-    reset_prob : float, optional
+    reset_probability : float, optional
         Probability that a random surfer jumps to an arbitrary page.
 
     threshold : float, optional
@@ -109,8 +117,6 @@ def create(graph, reset_prob=0.15,
     verbose : bool, optional
         If True, print progress updates.
 
-    plot : bool, optional
-        If True, display the progress plot.
 
     Returns
     -------
@@ -139,13 +145,12 @@ def create(graph, reset_prob=0.15,
     --------
     PagerankModel
     """
+    _mt._get_metric_tracker().track('toolkit.graph_analytics.pagerank.create')
+
     if not isinstance(graph, _SGraph):
         raise TypeError('graph input must be a SGraph object.')
 
-    if plot is True:
-        print "The plot functionality for pagerank is not yet implemented."
-
-    opts = {'threshold': threshold, 'reset_prob': reset_prob,
-            'max_iter': max_iterations, 'graph': graph.__proxy__}
-    params = _main.run('pagerank', opts, verbose, plot)
+    opts = {'threshold': threshold, 'reset_probability': reset_probability,
+            'max_iterations': max_iterations, 'graph': graph.__proxy__}
+    params = _main.run('pagerank', opts, verbose)
     return PagerankModel(params['model'])
