@@ -185,7 +185,8 @@ def _model_parameter_search(task):
     task.outputs['save_path'] = result_accumulator
 
 def model_parameter_search(environment, model_factory, train_set_path, save_path, test_set_path=None,
-                           standard_model_params={}, hyper_params=None, max_num_models='all', name=None):
+                           standard_model_params={}, hyper_params=None, max_num_models='all', name=None, 
+                           num_hosts=1):
     '''
     Searches for optimal model parameters. Automatically creates models using different parameters.
     Optionally, evaluates these models using a test set.
@@ -195,12 +196,15 @@ def model_parameter_search(environment, model_factory, train_set_path, save_path
     will be tried. If you do not specify `hyper_params`, a set of default parameter combinations,
     based on the model type, will be tried.
 
-    model_parameter_search is supported for: 
-    :py:class:`~graphlab.linear_regression.LinearRegressionModel`, 
-    :py:class:`~graphlab.logistic_regression.LogisticRegressionModel`,
-    :class:`~graphlab.recommender.MatrixFactorizationModel`, :py:class:`~graphlab.svm.SVMModel`,
+    model_parameter_search is supported for:
+    :py:class:`~graphlab.linear_regression.LinearRegression`,
+    :py:class:`~graphlab.boosted_trees_regression.BoostedTreesRegression`,
+    :py:class:`~graphlab.logistic_classifier.LogisticClassifier`,
+    :py:class:`~graphlab.svm_classifier.SVMClassifier`,
+    :py:class:`~graphlab.boosted_trees_classifier.BoostedTreesClassifier`,
+    :py:class:`~graphlab.neuralnet_classifier.NeuralNetClassifier`,
+    :py:class:`~graphlab.recommender.factorization_recommender.FactorizationRecommender`, and
     :py:class:`~graphlab.kmeans.KmeansModel` and 
-    :py:class:`~graphlab.tree_ensembles.boosted_trees.BoostedTreesModel`
 
     Parameters
     ----------
@@ -213,7 +217,7 @@ def model_parameter_search(environment, model_factory, train_set_path, save_path
     model_factory : function
         This is the function (always ending in "create") that you would normally use to create the
         model. For example, if you wanted to use `model_parameter_search` for
-        :py:class:`~graphlab.linear_regression.LinearRegressionModel`, you would set this parameter
+        :py:class:`~graphlab.linear_regression.LinearRegression`, you would set this parameter
         to `graphlab.linear_regression.create`.
 
     train_set_path : str
@@ -240,10 +244,10 @@ def model_parameter_search(environment, model_factory, train_set_path, save_path
         The keys in the dictionary must be the names of the `model_factory` parameters that you want
         to search over. The value for a key will depend on the type that `model_factory` expects for
         that parameter. If the type of the parameter is not a dictionary, use a list of all the
-        values you want to search over. For example if we we're using 
-        :class:`~graphlab.recommender.MatrixFactorizationModel` and want to search over all
-        `n_factors` from 10 to 15 and the `regularization` values of 1 and 5, you would use:
-        `{ 'n_factors': [10, 11, 12, 13, 14, 15], 'regularization': [1, 5] }`. If `model_factory`
+        values you want to search over. For example if we we're using
+        :class:`~graphlab.recommender.FactorizationRecommender` and want to search over all
+        `num_factors` from 10 to 15 and the `regularization` values of 1 and 5, you would use:
+        `{ 'num_factors': [10, 11, 12, 13, 14, 15], 'regularization': [1, 5] }`. If `model_factory`
         expects a dictionary for a parameter, the same rules apply recursively (see boosted trees
         example below).
 
@@ -256,6 +260,11 @@ def model_parameter_search(environment, model_factory, train_set_path, save_path
         The name for the :class:`Job <graphlab.deploy>` that will be created. If not specified then
         the name will be 'Model-Parameter-Search-(timestamp)'.
 
+    num_hosts : int, optional
+        The number of hosts to use when performing model parameter search on
+        EC2. If this model parameter search is to occur on Hadoop or locally,
+        this parameter is ignored.
+
     Returns
     -------
     out : :class:`job <graphlab.deploy>` object
@@ -273,25 +282,27 @@ def model_parameter_search(environment, model_factory, train_set_path, save_path
 
     .. sourcecode:: python
 
+        # SFrames need to be saved. Make sure all file paths are absolute.
         >>> train_file_path = '/data/train_set.gl'
         >>> test_file_path = '/data/test_set.gl'
         >>> test_set.save(test_file_path)
         >>> train_set.save(train_file_path)
 
+        # Setup a python environment
         >>> env = graphlab.deploy.environment.Local('local env')
 
         # For Kmeans using the default range of hyper-parameters for that model.
         >>> model_parameter_search(env, graphlab.kmeans.create, train_file_path, '/data/results.gl')
 
         # For Matrix Factorization with a target of 'rating', using the default range of hyper-params.
-        model_parameter_search(env, graphlab.recommender.create, train_file_path,
+        >>> model_parameter_search(env, graphlab.factorization_recommender.create, train_file_path,
                                 '/data/results.gl', test_set_path = test_file_path,
                                 standard_model_params = {'target': 'rating'}
                               )
 
 
-        # For booosted trees with a target of 'results', trying only a max_depth 5 and 10.
-        # This time the parameter search will be done asynchronously on an EC2 host. 
+        # For boosted trees with a target of 'results', trying only a max_depth 5 and 10.
+        # This time the parameter search will be done asynchronously on an EC2 host.
         >>> train_file_path = 's3://my-bucket-name/train_file_path'
         >>> test_file_path = 's3://my-bucket-name/test_file_path'
         >>> test_set.save(test_file_path)
@@ -299,12 +310,14 @@ def model_parameter_search(environment, model_factory, train_set_path, save_path
 
         >>> graphlab.aws.set_credentials('RBZH792CTQPP7T435BGQ', '7x2hMqplWsLpU/qQCN6xAPKcmWo46TlPJXYTvKcv')
         >>> ec2_env = graphlab.deploy.environment.EC2('ec2 env', 's3://my-bucket-name/log-dir')
-        >>> job = model_parameter_search(ec2_env, graphlab.boosted_trees.create, train_file_path,
+        >>> job = model_parameter_search(ec2_env, graphlab.boosted_trees_classifier.create, train_file_path,
                                           's3://my-bucket-name/save_file_path',
                                           test_set_path = test_file_path,
                                           standard_model_params = {'target': 'result'},
-                                          hyper_params = { 'params': {'max_depth': [5, 10] } }
+                                          hyper_params = {'max_depth': [5, 10]}
                                         )
+
+        # Get the status of the jobs
         >>> job.get_status()
     '''
     _get_metric_tracker().track('jobs.model_parameter_search')
@@ -312,6 +325,12 @@ def model_parameter_search(environment, model_factory, train_set_path, save_path
     if name is None:
         name =  "Model-Parameter-Search-%s" % _time()
 
+    # set num_hosts on environment if passed into model_parameter_search, only used by EC2 
+    if not isinstance(num_hosts, int):
+        raise TypeError('Invalid type for num_hosts parameter. Must be a positive integer')
+    environment = environment.clone() # clone 1st so not mutating what user passed in.
+    environment.num_hosts = num_hosts
+
     # Determine search space
     if(hyper_params is None):
         hyper_params = _get_default_parameter_range(model_factory, standard_model_params)
