@@ -15,21 +15,33 @@ make it easier to interactively create new features for Vowpal Wabbit models.
 
 import graphlab as _graphlab
 import graphlab.connect as _mt
-from graphlab.toolkits.model import Model
+from graphlab.toolkits._model import Model as _Model
 from graphlab.data_structures.sframe import SFrame as _SFrame
 from graphlab.data_structures.sarray import SArray as _SArray
 from graphlab.deps import pandas as _pandas, HAS_PANDAS as _HAS_PANDAS
-import time
+import time as _time
+from graphlab.toolkits._main import ToolkitError as _ToolkitError
+from graphlab.toolkits._model import _get_default_options_wrapper as \
+                                                  __get_default_options_wrapper
 
 
+get_default_options = __get_default_options_wrapper(
+                          'vowpal_wabbit', 
+                          'vowpal_wabbit', 
+                          'VowpalWabbitModel')
+
 def create(dataset, target,
-           loss_function='squared', quadratic=[],
+           loss_function='squared',
+           quadratic=[],
            l1_penalty=0.0, l2_penalty=0.0,
            bigram=False,
-           step_size=0.5, num_bits=18, verbose=False, max_iterations=1,
+           step_size=0.5, num_bits=18, verbose=False,
+           max_iterations=1,
            command_line_args=''):
     """
-    create(dataset, target, loss_function='squared', quadratic=list(), l1_penalty=0.0, l2_penalty=0.0, bigram=False, step_size=0.5, num_bits=18, verbose=False, max_iterations=1, command_line_args='')
+    create(dataset, target, loss_function='squared', quadratic=list(), 
+    l1_penalty=0.0, l2_penalty=0.0, bigram=False, step_size=0.5, num_bits=18, 
+    verbose=False, max_iterations=1, command_line_args='')
 
     Learn a large linear model using Vowpal Wabbit.
 
@@ -69,7 +81,7 @@ def create(dataset, target,
         The name of the column in ``dataset`` that is the prediction target.
         This column must have a numeric type.
 
-    loss_function : {'squared', 'hinge', 'logistic', 'quantile'}, optional    
+    loss_function : {'squared', 'hinge', 'logistic', 'quantile'}, optional
         This defines the `loss function
         <http://en.wikipedia.org/wiki/Loss_function>`_ used during optimization.
         Typical choices:
@@ -131,12 +143,12 @@ def create(dataset, target,
 
     Returns
     -------
-    out : VWModel
+    out : VowpalWabbitModel
         A model that can be used for predicting new cases.
 
     See Also
     --------
-    VWModel.predict, VWModel.evaluate
+    VowpalWabbitModel.predict, VowpalWabbitModel.evaluate
 
     Notes
     -----
@@ -167,8 +179,8 @@ def create(dataset, target,
 
     _mt._get_metric_tracker().track('toolkit.vowpal_wabbit.create')
 
-    if not (isinstance(dataset, _SFrame) or _HAS_PANDAS and isinstance(dataset, _pandas.DataFrame)):
-        raise TypeError("Input 'dataset' must be an SFrame or pandas.DataFrame")
+    if not (isinstance(dataset, _SFrame)):
+        raise TypeError("Input 'dataset' must be an SFrame")
 
     if type(dataset) != _SFrame:
         dataset = _SFrame(dataset)
@@ -193,19 +205,19 @@ def create(dataset, target,
             'extra_command_line_args': command_line_args}
 
     # Initialize the model with basic parameters
-    response = _graphlab.toolkits.main.run("vw_init", opts)
-    m = VWModel(response['model'])
+    response = _graphlab.toolkits._main.run("vw_init", opts)
+    m = VowpalWabbitModel(response['model'])
 
     # Train the model on the given data set and retrieve predictions
     opts = {'model': m.__proxy__,
             'data': dataset}
-    response = _graphlab.toolkits.main.run("vw_train", opts)
-    m = VWModel(response['model'])
+    response = _graphlab.toolkits._main.run("vw_train", opts)
+    m = VowpalWabbitModel(response['model'])
 
     yhat = _SArray(None, _proxy=response['predictions'])
 
     # Evaluate model
-    start_time = time.time()
+    start_time = _time.time()
     y = dataset[target]
 
     if loss_function == 'logistic':
@@ -213,26 +225,24 @@ def create(dataset, target,
         if not all(is_one_or_neg_one):
             raise TypeError('When using `logistic` as a loss function, the target column must contain only 1\'s and -1\'s.')
         y = y.apply(lambda x: int(x*.5 + .5))
-        m = m._set('train_accuracy', _graphlab.evaluation.accuracy(y, yhat))
-        m = m._set('confusion_table', _graphlab.evaluation.confusion_matrix(y, yhat))
+        m = m._set('training_accuracy', _graphlab.evaluation.accuracy(y, yhat))
     else:
-        m = m._set('train_rmse', _graphlab.evaluation.rmse(y, yhat))
-    m = m._set('train_stats_elapsed_time', time.time() - start_time)
-
+        m = m._set('training_rmse', _graphlab.evaluation.rmse(y, yhat))
     return m
 
 
-class VWModel(Model):
+class VowpalWabbitModel(_Model):
     """
     Wrapper around Vowpal Wabbit.
     """
 
     def __init__(self, model_proxy):
         self.__proxy__ = model_proxy
+        self.__name__ = 'vowpal_wabbit'
 
     def _get_wrapper(self):
         def model_wrapper(model_proxy):
-            return VWModel(model_proxy)
+            return VowpalWabbitModel(model_proxy)
         return model_wrapper
 
     def list_fields(self):
@@ -248,6 +258,7 @@ class VWModel(Model):
         See Also
         --------
         get
+ 
 
         Examples
         --------
@@ -258,14 +269,13 @@ class VWModel(Model):
         ['extra_command_line_args',
         'bigram',
         'target',
-        'train_stats_elapsed_time',
         'step_size',
         'l1_penalty',
         'elapsed_time',
         'cache_file',
         'final_regressor_file',
         'num_bits',
-        'train_rmse',
+        'training_rmse',
         'loss_function',
         'quadratic',
         'l2_penalty',
@@ -273,19 +283,19 @@ class VWModel(Model):
         'initial_regressor_file',
         'command_line_args',
         'max_iterations']
-        """ 
+        """
 
         _mt._get_metric_tracker().track('toolkit.vowpal_wabbit.list_fields')
-        opts = {'model': self.__proxy__}
-        response = _graphlab.toolkits.main.run("vw_get_model_description", opts)
-        fields = response
-        return fields.keys()
+        opts = {'model': self.__proxy__,
+                'model_name': self.__name__}
+        response = _graphlab.toolkits._main.run('vw_list_keys', opts)
+        return sorted(response.keys())
 
     def get(self, field):
         """
-        Return the value of a given field. The list of all queryable fields can 
+        Return the value of a given field. The list of all queryable fields can
         be obtained with the
-        :func:`~graphlab.vowpal_wabbit.VWModel.list_fields` method.
+        :func:`~graphlab.vowpal_wabbit.VowpalWabbitModel.list_fields` method.
 
         Parameters
         ----------
@@ -309,9 +319,17 @@ class VWModel(Model):
         >>> m.get('step_size')
         .5
         """
-        _mt._get_metric_tracker().track('toolkit.vowpal_wabbit.get_type')
-        opts = {'model': self.__proxy__}
-        response = _graphlab.toolkits.main.run("vw_get_model_description", opts)
+        _mt._get_metric_tracker().track('toolkit.vowpal_wabbit.get')
+
+        opts = {'model': self.__proxy__,
+                'model_name': self.__name__,
+                'field': field}
+        response = _graphlab.toolkits._main.run('vw_get_value',
+                                               opts)
+        return response['value']
+
+        opts = {'model': self.__proxy__, 'model_name': 'vowpal_wabbit'}
+        response = _graphlab.toolkits._main.run("vw_get_value", opts)
         fields = response
         return fields[field]
 
@@ -319,8 +337,8 @@ class VWModel(Model):
         opts = {'model': self.__proxy__,
                 'key': field,
                 'value': value}
-        response = _graphlab.toolkits.main.run("vw_set_model_description", opts)
-        return VWModel(response['model'])
+        response = _graphlab.toolkits._main.run("vw_set_model_description", opts)
+        return VowpalWabbitModel(response['model'])
 
     def __str__(self):
         """
@@ -331,33 +349,28 @@ class VWModel(Model):
         out : string
             A description of the NearestNeighborsModel.
         """
- 
-        opts = {'model': self.__proxy__}
-        fields = _graphlab.toolkits.main.run("vw_get_model_description", opts)
 
-        s = "Vowpal Wabbit Model: \n"
-        s += "\n   target column:  " + str(fields['target'])
-        s += "\n   loss function:  " + str(fields['loss_function'])
-        s += "\n   step size:      " + str(fields['step_size'])
-        s += "\n   L1 penalty:     " + str(fields['l1_penalty'])
-        s += "\n   L2 penalty:     " + str(fields['l2_penalty'])
-        s += "\n   verbose:        " + str(fields['verbose'])
-        s += "\n   bits:           " + str(fields['num_bits'])
-        s += "\n   max iterations: " + str(fields['max_iterations'])
+        opts = {'model': self.__proxy__,
+                'model_name': self.__name__}
+        fields = _graphlab.toolkits._main.run("vw_get_current_options", opts)
 
-        s += '\n'
-        for k in ['train_rmse', 'train_accuracy', 'train_stats_elapsed_time']:
-            if k in fields.keys():
-                s += "\n   {k}: {v}".format(k=k, v=fields[k])
+        model_fields = [
+            ("Target column", 'target'),
+            ("Loss function", 'loss_function'),
+            ("Step size",     'step_size'),
+            ("L1 penalty",    'l1_penalty'),
+            ("L2 penalty",    'l2_penalty'),
+            ("Verbose",       'verbose'),
+            ("Number of bits",'num_bits'),
+            ("Max iterations", 'max_iterations')]
 
-        if 'confusion_table' in fields.keys():
-            cf = self.get('confusion_table')
-            s += "\n     # true positives:  {a}".format(a=cf['true_positive'])
-            s += "\n     # false negatives: {a}".format(a=cf['false_negative'])
-            s += "\n     # true negatives:  {a}".format(a=cf['true_negative'])
-            s += "\n     # false positives: {a}".format(a=cf['false_positive'])
+        if 'training_rmse' in fields.keys():
+            model_fields.append(("Training RMSE", 'training_rmse'))
+        if 'training_accuracy' in fields.keys():
+            model_fields.append(("Training accuracy", 'training_accuracy'))
+        return _graphlab.toolkits._internal_utils._toolkit_repr_print(self, \
+                                                [model_fields], width=24)
 
-        return s
 
     def __repr__(self):
         """
@@ -376,8 +389,8 @@ class VWModel(Model):
         >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
         >>> data['price'] = data['price'].apply(lambda x: 1 if x > 30000 else -1)
         >>> m = graphlab.vowpal_wabbit.create(data, 'price')
-        >>> m.summary() 
-        Vowpal Wabbit Model: 
+        >>> m.summary()
+        Vowpal Wabbit Model:
             target column:  price
             loss function:  squared
             step size:      0.5
@@ -386,14 +399,22 @@ class VWModel(Model):
             verbose:        0
             bits:           18
             max iterations: 1
-            train_rmse: 0.414910189668
-            train_stats_elapsed_time: 0.00363111495972
+            training_rmse: 0.414910189668
         """
+        opts = {'model': self.__proxy__,
+                'model_name': self.__name__}
+
+        fields = _graphlab.toolkits._main.run("vw_get_current_options", opts)
+
+        print ""
+        print "                    Model summary                       "
+        print "--------------------------------------------------------"
 
         _mt._get_metric_tracker().track('toolkit.vowpal_wabbit.summary')
-        print self.__str__()
+        print self.__repr__()
 
-    def training_stats(self):
+
+    def _training_stats(self):
         """
         Get information about model creation, e.g. time elapsed during model
         fitting, data loading, and more.
@@ -412,19 +433,18 @@ class VWModel(Model):
         >>> data =  graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
         >>> data['price'] = data['price'].apply(lambda x: 1 if x > 30000 else -1)
         >>> m = graphlab.vowpal_wabbit.create(data, 'price')
-        >>> m.training_stats() 
+        >>> m._training_stats()
         {'elapsed_time': 0.006756,
-        'train_rmse': 0.4149101896680863,
-        'train_stats_elapsed_time': 0.003631114959716797}
+        'training_rmse': 0.4149101896680863}
         """
-        _mt._get_metric_tracker().track('toolkit.vowpal_wabbit.training_stats')
-        opts = {'model': self.__proxy__}
-        response = _graphlab.toolkits.main.run("vw_get_training_stats", opts)
+        _mt._get_metric_tracker().track('toolkit.vowpal_wabbit._training_stats')
+        opts = {'model': self.__proxy__, 'model_name': self.__name__}
+        response = _graphlab.toolkits._main.run("vw_training_stats", opts)
         return response
 
     def predict(self, dataset):
         """
-        Use the trained :class:`~graphlab.vowpal_wabbit.VWModel` to make
+        Use the trained :class:`~graphlab.vowpal_wabbit.VowpalWabbitModel` to make
         predictions about the target column that was provided during
         :func:`~graphlab.vowpal_wabbit.create`.
 
@@ -448,7 +468,7 @@ class VWModel(Model):
 
         opts = {'model': self.__proxy__,
                 'data': dataset}
-        response = _graphlab.toolkits.main.run("vw_predict", opts)
+        response = _graphlab.toolkits._main.run("vw_predict", opts)
 
         # Convert predictions to an SArray
         return _SArray(None, _proxy=response['predictions'])
@@ -475,11 +495,11 @@ class VWModel(Model):
 
             max\_error = \max_{i=1}^N \|\widehat{y}_i - y_i\| .
 
-        If the model is trained with ``logistic`` loss, then the model is 
-        evaluated as a classifier with a decision threshold of 0. The metrics 
-        are classification accuracy and confusion table.  Classification 
-        accuracy is the fraction of examples whose predicted and actual classes 
-        match. The confusion table contains the cross-tabulation of actual and 
+        If the model is trained with ``logistic`` loss, then the model is
+        evaluated as a classifier with a decision threshold of 0. The metrics
+        are classification accuracy and confusion matrix.  Classification
+        accuracy is the fraction of examples whose predicted and actual classes
+        match. The confusion matrix contains the cross-tabulation of actual and
         predicted classes for the target variable.
 
         Parameters
@@ -492,9 +512,9 @@ class VWModel(Model):
         Returns
         -------
         out : dict
-            Dictionary of evaluation results. For ``squared`` loss, the 
+            Dictionary of evaluation results. For ``squared`` loss, the
             dictionary keys are *rmse* and *max_error*.  For ``logistic`` loss,
-            the dictionary keys are *accuracy* and *confusion_table*.
+            the dictionary keys are *accuracy* and *confusion_matrix*.
 
         See Also
         --------
@@ -511,7 +531,7 @@ class VWModel(Model):
 
         target_column = self.get('target')
         if target_column not in dataset.column_names():
-            raise ToolkitError, \
+            raise _ToolkitError, \
                 "Input dataset must contain a target column for " \
                 "evaluation of prediction quality."
 
@@ -525,15 +545,17 @@ class VWModel(Model):
             return {'rmse': rmse,
                     'max_error': max_error}
         elif loss == 'logistic':
-            targets = targets.astype(int)
-            # Convert target to 0/1
-            targets = targets.apply(lambda x: 0 if x == -1 else x)
-            accuracy = _graphlab.evaluation.accuracy(targets, predictions, threshold=0)
-            confusion_table = _graphlab.evaluation.confusion_matrix(targets, predictions, threshold=0)
+            accuracy = _graphlab.evaluation.accuracy(targets, predictions)
+            confusion_matrix = _graphlab.evaluation.confusion_matrix(targets, \
+                                  predictions)
             return {'accuracy' : accuracy,
-                    'confusion_table' : confusion_table }
+                    'confusion_matrix' : confusion_matrix }
         else:
-            raise ToolkitError, "VW evaluate currently only supports models trained with squared or logistic loss."
-
+            raise _ToolkitError, "VW evaluate currently only supports models trained with squared or logistic loss."
 
+    @classmethod
+    def _get_queryable_methods(cls):
+        '''Returns a list of method names that are queryable through Predictive
+        Service'''
+        return {'predict':{'dataset':'sframe'}}
 
