@@ -1,28 +1,27 @@
 """
 This module defines top level utility functions for GraphLab.
 """
-import urllib
-import urllib2
-import sys
-import os
-import re
-from zipfile import ZipFile
-import bz2
-import tarfile
+import urllib as _urllib
+import urllib2 as _urllib2
+import sys as _sys
+import os as _os
+import re as _re
+from zipfile import ZipFile as _ZipFile
+import bz2 as _bz2
+import tarfile as _tarfile
 import ConfigParser as _ConfigParser
+import itertools as _itertools
 
 from graphlab.connect.aws._ec2 import get_credentials as _get_aws_credentials
-import graphlab.connect.main as glconnect
-import graphlab.connect.server as server
-
+import graphlab.connect as _mt
+import graphlab.connect.main as _glconnect
+import graphlab.connect.server as _server
 import graphlab.version_info
 from pkg_resources import parse_version
 
+import logging as _logging
 
-import logging
-
-__LOGGER__ = logging.getLogger(__name__)
-
+__LOGGER__ = _logging.getLogger(__name__)
 
 def make_internal_url(url):
     """
@@ -66,7 +65,7 @@ def make_internal_url(url):
             # protocol is a remote url not on server, just return
             return url
         elif protocol == 'hdfs':
-            if isinstance(glconnect.get_server(), server.LocalServer) and not server._get_hadoop_class_path():
+            if isinstance(_glconnect.get_server(), _server.LocalServer) and not _server._get_hadoop_class_path():
                 raise ValueError("HDFS URL is not supported because Hadoop not found. Please make hadoop available from PATH or set the environment variable HADOOP_HOME and try again.")
             else:
                 return url
@@ -83,7 +82,7 @@ def make_internal_url(url):
             path_on_server = path
         elif protocol == 'local':
         # url for files on local client, check if we are connecting to local server
-            if (isinstance(glconnect.get_server(), server.LocalServer)):
+            if (isinstance(_glconnect.get_server(), _server.LocalServer)):
                 path_on_server = path
             else:
                 raise ValueError('Cannot use local URL when connecting to a remote server.')
@@ -91,9 +90,9 @@ def make_internal_url(url):
             raise ValueError('Invalid url protocol %s. Supported url protocols are: remote://, local://, s3://, https:// and hdfs://' % protocol)
     elif len(urlsplit) == 1:
         # expand ~ to $HOME
-        url = os.path.expanduser(url) 
+        url = _os.path.expanduser(url)
         # url for files on local client, check if we are connecting to local server
-        if (isinstance(glconnect.get_server(), server.LocalServer)):
+        if (isinstance(_glconnect.get_server(), _server.LocalServer)):
             path_on_server = url
         else:
             raise ValueError('Cannot use local URL when connecting to a remote server.')
@@ -101,7 +100,7 @@ def make_internal_url(url):
         raise ValueError('Invalid url: %s' % url)
 
     if path_on_server:
-        return os.path.abspath(os.path.expanduser(path_on_server))
+        return _os.path.abspath(_os.path.expanduser(path_on_server))
     else:
         raise ValueError('Invalid url: %s' % url)
 
@@ -126,23 +125,23 @@ def download_dataset(url_str, extract=True, force=False, output_dir="."):
     """
     fname = output_dir + "/" + url_str.split("/")[-1]
     #download the file from the web
-    if not os.path.isfile(fname) or force:
+    if not _os.path.isfile(fname) or force:
         print "Downloading file from: ", url_str
-        urllib.urlretrieve(url_str, fname)
+        _urllib.urlretrieve(url_str, fname)
         if extract and fname[-3:] == "zip":
             print "Decompressing zip archive", fname
-            ZipFile(fname).extractall(output_dir)
+            _ZipFile(fname).extractall(output_dir)
         elif extract and fname[-6:] == ".tar.gz":
             print "Decompressing tar.gz archive", fname
-            tarfile.TarFile(fname).extractall(output_dir)
+            _tarfile.TarFile(fname).extractall(output_dir)
         elif extract and fname[-7:] == ".tar.bz2":
             print "Decompressing tar.bz2 archive", fname
-            tarfile.TarFile(fname).extractall(output_dir)
+            _tarfile.TarFile(fname).extractall(output_dir)
         elif extract and fname[-3:] == "bz2":
             print "Decompressing bz2 archive: ", fname
             outfile = open(fname.split(".bz2")[0], "w")
             print "Output file: ", outfile
-            for line in bz2.BZ2File(fname, "r"):
+            for line in _bz2.BZ2File(fname, "r"):
                 outfile.write(line)
             outfile.close()
     else:
@@ -162,15 +161,15 @@ def get_newest_version(timeout=5, _url=__GLCREATE_CURRENT_VERSION_URL__):
     url: string
         The URL to go to to check the current version.
     """
-    request = urllib2.urlopen(url=_url, timeout=timeout)
+    request = _urllib2.urlopen(url=_url, timeout=timeout)
     version = request.read()
     __LOGGER__.debug("current_version read %s" % version)
     return version
 
 
-def perform_version_check(configfile=(os.path.join(os.path.expanduser("~"), ".graphlab", "config")),
+def perform_version_check(configfile=(_os.path.join(_os.path.expanduser("~"), ".graphlab", "config")),
                           _url=__GLCREATE_CURRENT_VERSION_URL__,
-                          _outputstream=sys.stderr):
+                          _outputstream=_sys.stderr):
     """
     Checks if currently running version of GraphLab is less than the version
     available from graphlab.com. Prints a message if the graphlab.com servers
@@ -184,9 +183,8 @@ def perform_version_check(configfile=(os.path.join(os.path.expanduser("~"), ".gr
     """
     skip_version_check = False
     try:
-        if (os.path.isfile(configfile)):
-            import ConfigParser
-            config = ConfigParser.ConfigParser()
+        if (_os.path.isfile(configfile)):
+            config = _ConfigParser.ConfigParser()
             config.read(configfile)
             section = 'Product'
             key = 'skip_version_check'
@@ -217,7 +215,7 @@ def is_directory_archive(path):
     """
     Utiilty function that returns True if the path provided is a directory that has an SFrame or SGraph in it.
 
-    SFrames are written to disk as a directory archive, this function identifies if a given directory is an archive 
+    SFrames are written to disk as a directory archive, this function identifies if a given directory is an archive
     for an SFrame.
 
     Parameters
@@ -232,15 +230,15 @@ def is_directory_archive(path):
     if path is None:
         return False
 
-    if not os.path.isdir(path):
+    if not _os.path.isdir(path):
         return False
 
-    ini_path = os.path.join(path, 'dir_archive.ini')
+    ini_path = _os.path.join(path, 'dir_archive.ini')
 
-    if not os.path.exists(ini_path):
+    if not _os.path.exists(ini_path):
         return False
 
-    if os.path.isfile(ini_path):
+    if _os.path.isfile(ini_path):
         return True
 
     return False
@@ -263,7 +261,7 @@ def get_archive_type(path):
         raise TypeError('Unable to determine the type of archive at path: %s' % path)
 
     try:
-        ini_path = os.path.join(path, 'dir_archive.ini')
+        ini_path = _os.path.join(path, 'dir_archive.ini')
         parser = _ConfigParser.SafeConfigParser()
         parser.read(ini_path)
 
@@ -289,12 +287,12 @@ def get_environment_config():
     Parameters
     ----------
     None
-    
+
     Returns
     -------
     Returns a dictionary of {key:value,..}
     """
-    unity = glconnect.get_unity()
+    unity = _glconnect.get_unity()
     return unity.list_globals(False)
 
 def get_runtime_config():
@@ -306,20 +304,20 @@ def get_runtime_config():
     Parameters
     ----------
     None
-    
+
     Returns
     -------
     Returns a dictionary of {key:value,..}
     """
-    unity = glconnect.get_unity()
+    unity = _glconnect.get_unity()
     return unity.list_globals(True)
 
 def set_runtime_config(name, value):
     """
     Sets a runtime configuration value. These configuration values are also
-    read from environment variables at program startup if available. See 
+    read from environment variables at program startup if available. See
     :py:func:`graphlab.get_runtime_config()` to get the current values for
-    each variable. 
+    each variable.
 
     The default configuration is conservatively defined for machines with about
     4-8GB of RAM.
@@ -340,7 +338,7 @@ def set_runtime_config(name, value):
       GRAPHLAB_CACHE_FILE_LOCATIONS). On large systems increasing this as well
       as GRAPHLAB_FILEIO_MAXIMUM_CACHE_CAPACITY_PER_FILE can improve performance
       significantly. Defaults to 2147483648 bytes (2GB).
-      
+
     GRAPHLAB_FILEIO_MAXIMUM_CACHE_CAPACITY_PER_FILE:
       The maximum amount of memory which will be occupied by any individual
       intermediate SFrame/SArray. Once this limit is exceeded, the
@@ -348,17 +346,17 @@ def set_runtime_config(name, value):
       GRAPHLAB_CACHE_FILE_LOCATIONS). On large systems, increasing this as well
       as GRAPHLAB_FILEIO_MAXIMUM_CACHE_CAPACITY can improve performance
       significantly for large SFrames. Defaults to 134217728 bytes (128MB).
-      
+
     *Advanced Configuration Variables*
 
     GRAPHLAB_SFRAME_FILE_HANDLE_POOL_SIZE:
       The maximum number of file handles to use when reading SFrames/SArrays.
       Once this limit is exceeded, file handles will be recycled, reducing
-      performance. This limit should be rarely approached by most SFrame/SArray 
+      performance. This limit should be rarely approached by most SFrame/SArray
       operations. Large SGraphs however may create a large a number of SFrames
-      in which case increasing this limit may improve performance (You may 
+      in which case increasing this limit may improve performance (You may
       also need to increase the system file handle limit with "ulimit -n").
-      Defaults to 128.  
+      Defaults to 128.
 
     GRAPHLAB_SFRAME_IO_READ_LOCK
       Whether disk reads should be locked. Almost always necessary for magnetic
@@ -369,33 +367,104 @@ def set_runtime_config(name, value):
       The block size used by the SFrame file format. Increasing this will
       increase throughput of single SArray accesses, but decrease throughput of
       wide SFrame accesses. Defaults to 65536 bytes.
-      
+
     ----------
     name: A string referring to runtime configuration variable.
 
     value: The value to set the variable to.
-    
+
     Returns
     -------
-    Nothing 
+    Nothing
 
-    Raises 
+    Raises
     ------
     A RuntimeError if the key does not exist, or if the value cannot be
     changed to the requested value.
 
     """
-    unity = glconnect.get_unity()
+    unity = _glconnect.get_unity()
     ret = unity.set_global(name, value)
     if ret != "":
         raise RuntimeError(ret);
 
-GLOB_RE = re.compile("""[*?]""")
+GLOB_RE = _re.compile("""[*?]""")
 def split_path_elements(url):
-    parts = os.path.split(url)
+    parts = _os.path.split(url)
     m = GLOB_RE.search(parts[-1])
     if m:
         return (parts[0], parts[1])
     else:
         return (url, "")
 
+def validate_feature_types(dataset, features, valid_feature_types):
+    if features is not None:
+        if not hasattr(features, '__iter__'):
+            raise TypeError("Input 'features' must be an iterable type.")
+
+        if not all([isinstance(x, str) for x in features]):
+            raise TypeError("Input 'features' must contain only strings.")
+
+    ## Extract the features and labels
+    if features is None:
+        features = dataset.column_names()
+
+    col_type_map = {
+        col_name: col_type for (col_name, col_type) in
+        zip(dataset.column_names(), dataset.column_types()) }
+
+    valid_features = []
+    for col_name in features:
+        if col_type_map.get(col_name) in valid_feature_types:
+            valid_features.append(col_name)
+
+    if len(valid_features) == 0:
+        if features is not None:
+            raise TypeError("No valid feature types specified")
+        else:
+            raise TypeError("The dataset does not contain any valid feature types")
+
+    return features
+
+
+def crossproduct(d):
+    """
+    Create an SFrame containing the crossproduct of all provided options.
+
+    Parameters
+    ----------
+    d : dict
+        Each key is the name of an option, and each value is a list
+        of the possible values for that option.
+
+    Returns
+    -------
+    out : SFrame
+        There will be a column for each key in the provided dictionary,
+        and a row for each unique combination of all values.
+
+    Example
+    -------
+    settings = {'argument_1':[0, 1],
+                'argument_2':['a', 'b', 'c']}
+    print crossproduct(settings)
+    +------------+------------+
+    | argument_2 | argument_1 |
+    +------------+------------+
+    |     a      |     0      |
+    |     a      |     1      |
+    |     b      |     0      |
+    |     b      |     1      |
+    |     c      |     0      |
+    |     c      |     1      |
+    +------------+------------+
+    [6 rows x 2 columns]
+    """
+
+    _mt._get_metric_tracker().track('util.crossproduct')
+    from graphlab import SArray
+    d = [zip(d.keys(), x) for x in _itertools.product(*d.values())]
+    sa = [{k:v for (k,v) in x} for x in d]
+    return SArray(sa).unpack(column_name_prefix='')
+
+
