@@ -1,10 +1,6 @@
 from config import DEFAULT_CONFIG as CONFIG
 from metric_mock import MetricMock
 
-# sets default timeout to 3 seconds
-import socket
-socket.setdefaulttimeout(3)
-
 # metrics libraries
 import mixpanel
 import librato
@@ -20,7 +16,7 @@ import uuid
 __ALL__ = [ 'MetricTracker' ]
 
 try:
-  from psutil import TOTAL_PHYMEM, NUM_CPUS
+  from graphlab_psutil import TOTAL_PHYMEM, NUM_CPUS
 except ImportError:
   TOTAL_PHYMEM = 0
   NUM_CPUS = 0
